#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct


from qiling.os.windows.fncc import *
from qiling.os.const import *
from qiling.os.windows.utils import *
from qiling.os.windows.const import *
from qiling.const import *
from qiling.os.windows.structs import *
from .const import *

dllname = 'user32_dll'

#ATOM RegisterClassExA(
#  const WNDCLASSEXA *Arg1
#);
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={'lpWndClass': 'POINTER'})
def hook_RegisterClassExA(ql, address, params):
    ret = 0
    return ret


#BOOL UpdateWindow(
#  HWND hWnd
#);
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_UpdateWindow(ql, address, params):
    ret = 0
    return ret

# HWND CreateWindowExA(
#  DWORD     dwExStyle,
#  LPCSTR    lpClassName,
#  LPCSTR    lpWindowName,
#  DWORD     dwStyle,
#  int       X,
#  int       Y,
#  int       nWidth,
#  int       nHeight,
#  HWND      hWndParent,
#  HMENU     hMenu,
#  HINSTANCE hInstance,
#  LPVOID    lpParam
#);
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={'hMenu': 'POINTER'})
def hook_CreateWindowExA(ql, address, params):
    ret = 0
    return ret

# INT_PTR DialogBoxParamA(
#   HINSTANCE hInstance,
#   LPCSTR    lpTemplateName,
#   HWND      hWndParent,
#   DLGPROC   lpDialogFunc,
#   LPARAM    dwInitParam
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'LPCSTR': 'POINTER'})
def hook_DialogBoxParamA(ql, address, params):
    ret = 0
    return ret


# UINT GetDlgItemTextA(
# 	HWND  hDlg,
# 	int   nIDDlgItem,
# 	LPSTR lpString,
# 	int   cchMax
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetDlgItemTextA(ql, address, params):
    ret = 0
    hDlg = params["hDlg"]
    nIDDlgItem = params["nIDDlgItem"]
    lpString = params["lpString"]
    cchMax = params["cchMax"]

    ql.os.stdout.write(b"Input DlgItemText :\n")
    string = ql.os.stdin.readline().strip()[:cchMax]
    ret = len(string)
    ql.mem.write(lpString, string)

    return ret


# BOOL EndDialog(
#   HWND    hDlg,
#   INT_PTR nResult
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_EndDialog(ql, address, params):
    ret = 1
    return ret


# HWND GetDesktopWindow((
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetDesktopWindow(ql, address, params):
    pass


# BOOL OpenClipboard(
#  HWND hWndNewOwner
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_OpenClipboard(ql, address, params):
    return ql.os.clipboard.open(params['hWndNewOwner'])


# BOOL CloseClipboard();
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_CloseClipboard(ql, address, params):
    return ql.os.clipboard.close()


# HANDLE SetClipboardData(
#  UINT   uFormat,
#  HANDLE hMem
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_SetClipboardData(ql, address, params):
    try:
        data = bytes(params['hMem'], 'ascii', 'ignore')
    except (UnicodeEncodeError, TypeError):
        data = b""
    return ql.os.clipboard.set_data(params['uFormat'], data)


# HANDLE GetClipboardData(
#  UINT uFormat
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetClipboardData(ql, address, params):
    data = ql.os.clipboard.get_data(params['uFormat'])
    if data:
        addr = ql.os.heap.alloc(len(data))
        ql.mem.write(addr, data)
        return addr
    else:
        ql.log.debug('Failed to get clipboard data')
        return 0


# BOOL IsClipboardFormatAvailable(
#  UINT format
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_IsClipboardFormatAvailable(ql, address, params):
    rtn = ql.os.clipboard.format_available(params['uFormat'])
    return rtn


# UINT MapVirtualKeyW(
#   UINT uCode,
#   UINT uMapType
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_MapVirtualKeyW(ql, address, params):
    map_value = params["uMapType"]
    code_value = params["uCode"]
    map_dict = MAP_VK.get(map_value, None)
    if map_dict is not None:
        code = map_dict.get(code_value, None)
        if code is not None:
            return code
        else:
            ql.log.debug("Code value %x" % code_value)
            raise QlErrorNotImplemented("API not implemented")
    else:
        ql.log.debug("Map value %x" % map_value)
        raise QlErrorNotImplemented("API not implemented")


# SHORT GetKeyState(
#   int nVirtKey
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'int': 'UINT'})
def hook_GetKeyState(ql, address, params):
    let = chr(params["nVirtKey"])
    ql.log.debug(let)
    UP = 2
    DOWN = 0
    return UP


# UINT RegisterWindowMessageA(
#   LPCSTR lpString
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegisterWindowMessageA(ql, address, params):
    return hook_RegisterWindowMessageW.__wrapped__(ql, address, params)


# UINT RegisterWindowMessageW(
#   LPCWSTR lpString
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegisterWindowMessageW(ql, address, params):
    # maybe some samples really use this and we need to have a real implementation
    return 0xD10C


# HWND GetActiveWindow();
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetActiveWindow(ql, address, params):
    # maybe some samples really use this and we need to have a real implementation
    return 0xD10C


# HWND GetLastActivePopup(
#   HWND hWnd
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetLastActivePopup(ql, address, params):
    hwnd = params["hWnd"]
    return hwnd


# BOOL GetPhysicalCursorPos(
#   LPPOINT lpPoint
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetPhysicalCursorPos(ql, address, params):
    return 1


# int GetSystemMetrics(
#   int nIndex
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetSystemMetrics(ql, address, params):
    info = params["nIndex"]
    if info == SM_CXICON or info == SM_CYICON:
        # Size of icon
        return 32
    elif info == SM_CXVSCROLL:
        return 4
    elif info == SM_CYHSCROLL:
        return 300
    else:
        ql.log.debug("Info value %x" % info)
        raise QlErrorNotImplemented("API not implemented")


# HDC GetDC(
#   HWND hWnd
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetDC(ql, address, params):
    handler = params["hWnd"]
    # Maybe we should really emulate the handling of screens and windows. Is going to be a pain
    return 0xD10C


# int GetDeviceCaps(
#   HDC hdc,
#   int index
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetDeviceCaps(ql, address, params):
    # Maybe we should really emulate the handling of screens and windows. Is going to be a pain
    return 1


# int ReleaseDC(
#   HWND hWnd,
#   HDC  hDC
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'HWND': 'POINTER'})
def hook_ReleaseDC(ql, address, params):
    return 1


# DWORD GetSysColor(
#   int nIndex
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetSysColor(ql, address, params):
    info = params["nIndex"]
    return 0


# HBRUSH GetSysColorBrush(
#   int nIndex
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetSysColorBrush(ql, address, params):
    info = params["nIndex"]
    return 0xd10c


# HCURSOR LoadCursorA(
#   HINSTANCE hInstance,
#   LPCSTR    lpCursorName
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'HINSTANCE': 'POINTER', 'LPCSTR': 'INT'})
def hook_LoadCursorA(ql, address, params):
    return 0xd10c


# HCURSOR LoadCursorFromFileA(
#   LPCSTR lpFileName
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_LoadCursorFromFileA(ql, address, params):
    return hook_LoadCursorFromFileW.__wrapped__(ql, address, params)


# HCURSOR LoadCursorFromFileW(
#   LPCSTR lpFileName
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_LoadCursorFromFileW(ql, address, params):
    handle = Handle()
    ql.os.handle_manager.append(handle)
    return handle.id


# UINT GetOEMCP();
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetOEMCP(ql, address, params):
    return OEM_US


# int LoadStringW(
#   HINSTANCE hInstance,
#   UINT      uID,
#   LPSTR     lpBuffer,
#   int       cchBufferMax
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'HINSTANCE': 'POINTER'})
def hook_LoadStringW(ql, address, params):
    dst = params["lpBuffer"]
    max_len = params["cchBufferMax"]
    # FIXME, should not be hardcoded
    string = "AAAABBBBCCCCDDDD" + "\x00"
    if max_len == 0:
        if len(string) >= max_len:
            string[max_len] = "\x00"
            string = string[:max_len]
        ql.mem.write(dst, string.encode("utf-16le"))
    # should not count the \x00 byte
    return len(string) - 1


# int LoadStringA(
#   HINSTANCE hInstance,
#   UINT      uID,
#   LPSTR     lpBuffer,
#   int       cchBufferMax
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'HINSTANCE': 'POINTER'})
def hook_LoadStringA(ql, address, params):
    dst = params["lpBuffer"]
    max_len = params["cchBufferMax"]
    # FIXME, should not be hardcoded
    string = "AAAABBBBCCCCDDDD" + "\x00"
    if max_len == 0:
        if len(string) >= max_len:
            string[max_len] = "\x00"
            string = string[:max_len]
        ql.mem.write(dst, string.encode())
    # should not count the \x00 byte
    return len(string) - 1


# BOOL MessageBeep(
#   UINT uType
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_MessageBeep(ql, address, params):
    return 1


# HHOOK SetWindowsHookExA(
#   int       idHook,
#   HOOKPROC  lpfn,
#   HINSTANCE hmod,
#   DWORD     dwThreadId
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'HINSTANCE': 'POINTER'})
def hook_SetWindowsHookExA(ql, address, params):
    # Should hook a procedure to a dll
    hook = params["lpfn"]
    return hook


# BOOL UnhookWindowsHookEx(
#   HHOOK hhk
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_UnhookWindowsHookEx(ql, address, params):
    return 1


# BOOL ShowWindow(
#   HWND hWnd,
#   int  nCmdShow
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'HWND': 'POINTER'})
def hook_ShowWindow(ql, address, params):
    # return value depends on sample goal (evasion on just display error)
    return 0x1


# HICON LoadIconA(
#   HINSTANCE hInstance,
#   LPCSTR    lpIconName
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'HINSTANCE': 'POINTER', 'LPCSTR': 'UINT'})
def hook_LoadIconA(ql, address, params):
    return hook_LoadIconW.__wrapped__(ql, address, params)


# HICON LoadIconW(
#   HINSTANCE hInstance,
#   LPCWSTR    lpIconName
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'HINSTANCE': 'POINTER', 'LPCWSTR': 'UINT'})
def hook_LoadIconW(ql, address, params):
    inst, name = params

    if name not in (IDI_APPLICATION, IDI_ASTERISK, IDI_ERROR, IDI_EXCLAMATION, IDI_HAND,
                        IDI_INFORMATION, IDI_QUESTION, IDI_SHIELD, IDI_WARNING, IDI_WINLOGO):
        return 0
    return 1


# BOOL IsWindow(
#   HWND hWnd
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'HWND': 'POINTER'})
def hook_IsWindow(ql, address, params):
    # return value depends on sample  goal (evasion on just display error)
    return 0x1


# LRESULT SendMessageA(
#   HWND   hWnd,
#   UINT   Msg,
#   WPARAM wParam,
#   LPARAM lParam
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'HWND': 'POINTER', 'LPARAM': 'UINT'})
def hook_SendMessageA(ql, address, params):
    # TODO don't know how to get right return value
    return 0xD10C


# LRESULT LRESULT DefWindowProcA(
#   HWND   hWnd,
#   UINT   Msg,
#   WPARAM wParam,
#   LPARAM lParam
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'HWND': 'POINTER', 'LPARAM': 'UINT'})
def hook_DefWindowProcA(ql, address, params):
    # TODO don't know how to get right return value
    return 0xD10C


# LPWSTR CharNextW(
#   LPCWSTR lpsz
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'LPCWSTR': 'POINTER'})
def hook_CharNextW(ql, address, params):
    # Return next char if is different from \x00
    point = params["lpsz"][0]
    string = ql.os.read_wstring(point)
    params["lpsz"] = string
    if len(string) == 0:
        return point
    else:
        return point + 1


# LPWSTR CharNextA(
#   LPCSTR lpsz
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_CharNextA(ql, address, params):
    # Return next char if is different from \x00
    point = params["lpsz"][0]
    string = ql.os.read_cstring(point)
    params["lpsz"] = string
    if len(string) == 0:
        return point
    else:
        return point + 1


# LPWSTR CharPrevW(
#   LPCWSTR lpszStart,
#   LPCWSTR lpszCurrent
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'LPCWSTR': 'POINTER'})
def hook_CharPrevW(ql, address, params):
    # Return next char if is different from \x00
    current = params["lpszCurrent"]
    strcur = ql.os.read_wstring(current)
    start = params["lpszStart"]
    strstart = ql.os.read_wstring(start)
    params["lpszStart"] = strstart
    params["lpszCurrent"] = strcur

    if start == current:
        return start
    return current - 1


# LPWSTR CharPrevA(
#   LPCSTR lpszStart,
#   LPCSTR lpszCurrent
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'LPCSTR': 'POINTER'})
def hook_CharPrevA(ql, address, params):
    # Return next char if is different from \x00
    current = params["lpszCurrent"]
    strcur = ql.os.read_cstring(current)
    start = params["lpszStart"]
    strstart = ql.os.read_cstring(start)
    params["lpszStart"] = strstart
    params["lpszCurrent"] = strcur

    if start == current:
        return start
    return current - 1


# int WINAPIV wsprintfW(
#   LPWSTR  ,
#   LPCWSTR ,
#   ...
# );
@winsdkapi(cc=CDECL, dllname=dllname, param_num=3)
def hook_wsprintfW(ql, address, params):
    dst, p_format = ql.os.get_function_param(2)

    format_string = ql.os.read_wstring(p_format)
    count = format_string.count('%')
    args = ql.os.get_function_param(2 + count)[2:]
    size, string = ql.os.printf(address, format_string, args, "wsprintfW", wstring=True)

    if ql.archtype == QL_ARCH.X8664:
        # We must pop the stack correctly
        raise QlErrorNotImplemented("API not implemented")

    ql.mem.write(dst, (string + "\x00").encode("utf-16le"))
    return size


# int WINAPIV sprintf(
#   LPWSTR  ,
#   LPCWSTR ,
#   ...
# );
@winsdkapi(cc=CDECL, dllname=dllname, param_num=3)
def hook_sprintf(ql, address, params):
    dst, p_format = ql.os.get_function_param(2)
    format_string = ql.os.read_wstring(p_format)
    count = format_string.count('%')
    args = ql.os.get_function_param(2 + count)[2:]
    size, string = ql.os.printf(address, format_string, args, "sprintf", wstring=True)

    if ql.archtype == QL_ARCH.X8664:
        # We must pop the stack correctly
        raise QlErrorNotImplemented("API not implemented")

    ql.mem.write(dst, (string + "\x00").encode("utf-16le"))
    return size


# HWND GetForegroundWindow();
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetForegroundWindow(ql, address, params):
    return 0xF02E620D  # Value so we can recognize inside dumps


# BOOL MoveWindow(
#   HWND hWnd,
#   int  X,
#   int  Y,
#   int  nWidth,
#   int  nHeight,
#   BOOL bRepaint
# )
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_MoveWindow(ql, address, params):
    return 1


# int GetKeyboardType(
#  int nTypeFlag
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'int': 'UINT'})
def hook_GetKeyboardType(ql, address, params):
    """
    See https://salsa.debian.org/wine-team/wine/-/blob/master/dlls/user32/input.c
    """
    _type = params['nTypeFlag']
    if _type == 0:  # 0: Keyboard Type, 1: Keyboard subtype, 2: num func keys
        return 7
    elif _type == 1:
        return 0
    elif _type == 2:
        return 12
    return 0

# int wvsprintfA(
# LPTSTR lpOutput, 
# LPCTSTR lpFormat, 
# va_list ArgList
# );
@winsdkapi(cc=CDECL, dllname=dllname, replace_params_type={            
            "lpOutput": POINTER,
            "lpFormat": POINTER,
            "ArgList": POINTER,
        })
def hook_wvsprintfA(ql, address, params):
    return None


# int wsprintfA(
#    char *buffer,
#    const char *format,
# );
@winsdkapi(cc=CDECL, dllname=dllname, param_num=3)
def hook_wsprintfA(ql, address, params):
    dst, p_format, p_args = ql.os.get_function_param(3)
    format_string = ql.os.read_cstring(p_format)
    count = format_string.count('%')
    args = ql.os.get_function_param(2 + count)[2:]
    size, string = ql.os.printf(address, format_string, args, "wsprintfA")

    if ql.archtype== QL_ARCH.X8664:
        # We must pop the stack correctly
        raise QlErrorNotImplemented("API not implemented")

    ql.mem.write(dst, (string + "\x00").encode("utf-8"))
    return size

# int MessageBoxW(
#   HWND    hWnd,
#   LPCWSTR lpText,
#   LPCWSTR lpCaption,
#   UINT    uType
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_MessageBoxW(ql, address, params):
    # We always return a positive result
    type_box = params["uType"]
    if type_box == MB_OK or type_box == MB_OKCANCEL:
        return IDOK
    if type_box == MB_YESNO or type_box == MB_YESNOCANCEL:
        return IDYES
    else:
        ql.log.debug(type_box)
        raise QlErrorNotImplemented("API not implemented")


# int MessageBoxA(
#   HWND    hWnd,
#   LPCWSTR lpText,
#   LPCWSTR lpCaption,
#   UINT    uType
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_MessageBoxA(ql, address, params):
    return hook_MessageBoxW.__wrapped__(ql, address, params)


# BOOL GetCursorPos(
#   LPPOINT lpPoint
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetCursorPos(ql, address, params):
    # TODO maybe we can add it to the profile too
    p = Point(ql, 50, 50)
    dest = params["lpPoint"]
    p.write(dest)
    return 0


# HANDLE CreateActCtxW(
#   PCACTCTXW pActCtx
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_CreateActCtxW(ql, address, params):
    # TODo maybe is necessary to really create this
    addr = params["pActCtx"]
    handle = Handle(name="actctx")
    ql.os.handle_manager.append(handle)
    return handle.id


# DWORD GetWindowThreadProcessId(
#   HWND    hWnd,
#   LPDWORD lpdwProcessId
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetWindowThreadProcessId(ql, address, params):
    target = params["hWnd"]
    if target == ql.os.profile.getint("KERNEL", "pid") or target == ql.os.profile.getint("KERNEL", "shell_pid"):
        pid = ql.os.profile.getint("KERNEL", "parent_pid")
    else:
        raise QlErrorNotImplemented("API not implemented")
    dst = params["lpdwProcessId"]
    if dst != 0:
        ql.mem.write(dst, pid.to_bytes(4, "little"))
    return pid


# HWND GetShellWindow();
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetShellWindow(ql, address, params):
    return ql.os.profile.getint("KERNEL", "shell_pid")



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from qiling.os.const import *
from qiling.os.uefi.const import *
from ..fncc import *
from ..ProcessorBind import *
from ..UefiBaseType import *
from ..PiMultiPhase import *
from ..utils import write_int64, read_int64

# @see: MdePkg\Include\Pi\PiMultiPhase.h
class EFI_MMRAM_DESCRIPTOR(STRUCT):
	_fields_ = [
		('PhysicalStart',	EFI_PHYSICAL_ADDRESS),
		('CpuStart',		EFI_PHYSICAL_ADDRESS),
		('PhysicalSize',	UINT64),
		('RegionState',		UINT64)
	]

# @see: MdePkg\Include\Protocol\MmAccess.h
class EFI_SMM_ACCESS2_PROTOCOL(STRUCT):
	EFI_SMM_ACCESS2_PROTOCOL = STRUCT

	_fields_ = [
		('Open',			FUNCPTR(EFI_STATUS, PTR(EFI_SMM_ACCESS2_PROTOCOL))),
		('Close',			FUNCPTR(EFI_STATUS, PTR(EFI_SMM_ACCESS2_PROTOCOL))),
		('Lock',			FUNCPTR(EFI_STATUS, PTR(EFI_SMM_ACCESS2_PROTOCOL))),
		('GetCapabilities',	FUNCPTR(EFI_STATUS, PTR(EFI_SMM_ACCESS2_PROTOCOL), PTR(UINTN), PTR(EFI_MMRAM_DESCRIPTOR))),
		('LockState',		BOOLEAN),
		('OpenState',		BOOLEAN),
		('PADDING_0',		CHAR8 * 6)
	]

@dxeapi(params = {
	"This" : POINTER
})
def hook_Open(ql, address, params):
	ql.loader.smm_context.tseg_open = True

	return EFI_SUCCESS

@dxeapi(params = {
	"This" : POINTER
})
def hook_Close(ql, address, params):
	ql.loader.smm_context.tseg_open = False

	return EFI_SUCCESS

@dxeapi(params = {
	"This" : POINTER
})
def hook_Lock(ql, address, params):
	ql.loader.smm_context.tseg_locked = True

	return EFI_SUCCESS

def _coalesce(seq):
	"""Coalesce adjacent ranges on list, as long as they share the
	same attributes.
	"""

	res = []
	curr = seq[0]

	for item in seq[1:]:
		start, end, attr = item

		if start == curr[1] and attr == curr[2]:
			curr[1] = end
		else:
			res.append(curr)
			curr = item

	res.append(curr)

	return res

@dxeapi(params = {
	"This"          : POINTER,	# PTR(EFI_SMM_ACCESS2_PROTOCOL)
	"MmramMapSize"  : POINTER,	# IN OUT PTR(UINTN)
	"MmramMap"      : POINTER	# OUT PTR(EFI_MMRAM_DESCRIPTOR)
})
def hook_GetCapabilities(ql, address, params):
	heap = ql.loader.smm_context.heap

	# get a copy of smm heap chunks list sorted by starting address
	chunks = sorted(heap.chunks, key=lambda c: c.address)

	# turn chunks objects into 3-item entries: [start, end, inuse]
	chunks = [[ch.address, ch.address + ch.size, ch.inuse] for ch in chunks]

	# if first chunk does not start at heap start, add a dummy free chunk there
	if chunks[0][0] != heap.start_address:
		chunks.insert(0, [heap.start_address, chunks[0].address, False])

	# if last chunk does not end at heap end, add a dummy free chunk there
	if (chunks[-1][1]) != heap.end_address:
		chunks.append([chunks[-1][1], heap.end_address, False])

	# coalesce adjacent free / used chunks on the list
	chunks = _coalesce(chunks)

	size = len(chunks) * EFI_SMRAM_DESCRIPTOR.sizeof()
	MmramMapSize = params["MmramMapSize"]

	if read_int64(ql, MmramMapSize) < size:
		write_int64(ql, MmramMapSize, size)
		return EFI_BUFFER_TOO_SMALL

	MmramMap = params["MmramMap"]

	state = EFI_CACHEABLE
	state |= EFI_SMRAM_OPEN if ql.loader.smm_context.tseg_open else EFI_SMRAM_CLOSED
	state |= EFI_SMRAM_LOCKED if ql.loader.smm_context.tseg_locked else 0

	for i, ch in enumerate(chunks):
		desc = EFI_SMRAM_DESCRIPTOR()
		desc.PhysicalStart = ch[0]
		desc.CpuStart = ch[0]
		desc.PhysicalSize = ch[1] - ch[0]
		desc.RegionState = state | (EFI_ALLOCATED if ch[2] else 0)

		desc.saveTo(ql, MmramMap + (i * desc.sizeof()))

	return EFI_SUCCESS

descriptor = {
	"guid" : "c2702b74-800c-4131-8746-8fb5b89ce4ac",
	"struct" : EFI_SMM_ACCESS2_PROTOCOL,
	"fields" : (
		("Open",			hook_Open),
		("Close",			hook_Close),
		("Lock",			hook_Lock),
		("GetCapabilities",	hook_GetCapabilities)
	)
}



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from unicorn import *
from unicorn.x86_const import *

from struct import pack

from .arch import QlArch
from .x86_const import *
from qiling.const import *
from qiling.exception import *

class QlArchA8086(QlArch):
    def __init__(self, ql):
        super(QlArchA8086, self).__init__(ql)
        arch_8086_register_mappings = [
            reg_map_8, reg_map_16, reg_map_misc
        ]

        for reg_maper in arch_8086_register_mappings:
            self.ql.reg.expand_mapping(reg_maper)

        self.ql.reg.create_reverse_mapping()
        self.ql.reg.register_pc(UC_X86_REG_IP)
        self.ql.reg.register_sp(UC_X86_REG_SP)


    def stack_push(self, value):
        self.ql.reg.sp -= 2
        self.ql.mem.write(self.ql.reg.sp , self.ql.pack16(value))
        return self.ql.reg.sp


    def stack_pop(self):
        data = self.ql.unpack16(self.ql.mem.read(self.ql.reg.sp, 2))
        self.ql.reg.sp += 2
        return data


    def stack_read(self, offset):
        return self.ql.unpack32(self.ql.mem.read(self.ql.reg.sp+offset, 2))


    def stack_write(self, offset, data):
        return self.ql.mem.write(self.ql.reg.sp + offset, self.ql.pack16(data))


    def get_init_uc(self):
        uc = Uc(UC_ARCH_X86, UC_MODE_16)  
        return uc


#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

# basic values
PAGE_SIZE                   = 0x1000
VMMAP_PAGE_SIZE             = 0x100000
MAX_FD_SIZE					= 0xFF
MAX_PATH_SIZE               = 0x800

# GS
MSR_KERNEL_GS_BASE          = 0xc0000102

# kernel flags
KERN_SUCCESS                = 0
KERN_INVALID_ADDRESS        = 1
KERN_PROTECTION_FAILURE     = 2
KERN_NO_SPACE               = 3
KERN_INVALID_ARGUMENT       = 4
KERN_FAILURE                = 5
KERN_RESOURCE_SHORTAGE      = 6
KERN_NOT_RECEIVER           = 7
KERN_NO_ACCESS              = 8
KERN_MEMORY_FAILURE         = 9
KERN_MEMORY_ERROR           = 10
KERN_ALREADY_IN_SET         = 11
KERN_NOT_IN_SET             = 12
KERN_NAME_EXISTS            = 13
KERN_ABORTED                = 14
KERN_INVALID_NAME           = 15
KERN_INVALID_TASK           = 16
KERN_INVALID_RIGHT          = 17
KERN_INVALID_VALUE          = 18
KERN_UREFS_OVERFLOW         = 19
KERN_INVALID_CAPABILITY     = 20
KERN_RIGHT_EXISTS           = 21
KERN_INVALID_HOST           = 22
KERN_MEMORY_PRESENT         = 23
KERN_MEMORY_DATA_MOVED      = 24
KERN_MEMORY_RESTART_COPY    = 25
KERN_INVALID_PROCESSOR_SET  = 26
KERN_POLICY_LIMIT           = 27
KERN_INVALID_POLICY         = 28
KERN_INVALID_OBJECT         = 29
KERN_ALREADY_WAITING        = 30
KERN_DEFAULT_SET            = 31
KERN_EXCEPTION_PROTECTED    = 32
KERN_INVALID_LEDGER         = 33
KERN_INVALID_MEMORY_CONTROL = 34
KERN_INVALID_SECURITY       = 35
KERN_NOT_DEPRESSED          = 36
KERN_TERMINATED             = 37
KERN_LOCK_SET_DESTROYED     = 38
KERN_LOCK_UNSTABLE          = 39
KERN_LOCK_OWNED             = 40
KERN_LOCK_OWNED_SELF        = 41
KERN_SEMAPHORE_DESTROYED    = 42
KERN_RPC_SERVER_TERMINATED  = 43
KERN_RPC_TERMINATE_ORPHAN   = 44
KERN_RPC_CONTINUE_ORPHAN    = 45
KERN_NOT_SUPPORTED          = 46
KERN_NODE_DOWN              = 47
KERN_NOT_WAITING            = 48
KERN_OPERATION_TIMED_OUT    = 49
KERN_CODESIGN_ERROR         = 50
KERN_POLICY_STATIC          = 51
KERN_INSUFFICIENT_BUFFER_SIZE  = 52
KERN_RETURN_MAX             = 0x100

# CPU types 
CPU_ARCH_ABI64      =   0x01000000
CPU_TYPE_X86        =   7
CPU_TYPE_I386       =   CPU_TYPE_X86
CPU_TYPE_X86_64     =   CPU_ARCH_ABI64 | CPU_TYPE_X86

# mach mag flags
MACH_MSG_SUCCESS                =   0x00000000
MACH_MSG_MASK			        =   0x00003e00
MACH_MSG_IPC_SPACE		        =   0x00002000
MACH_MSG_VM_SPACE		        =   0x00001000
MACH_MSG_IPC_KERNEL		        =   0x00000800
MACH_MSG_VM_KERNEL		        =   0x00000400
MACH_SEND_IN_PROGRESS		    =   0x10000001
MACH_SEND_INVALID_DATA		    =   0x10000002
MACH_SEND_INVALID_DEST		    =   0x10000003
MACH_SEND_TIMED_OUT		        =   0x10000004
MACH_SEND_INVALID_VOUCHER	    =   0x10000005
MACH_SEND_INTERRUPTED		    =   0x10000007
MACH_SEND_MSG_TOO_SMALL		    =   0x10000008
MACH_SEND_INVALID_REPLY		    =   0x10000009
MACH_SEND_INVALID_RIGHT		    =   0x1000000a
MACH_SEND_INVALID_NOTIFY	    =   0x1000000b
MACH_SEND_INVALID_MEMORY	    =   0x1000000c
MACH_SEND_NO_BUFFER		        =   0x1000000d
MACH_SEND_TOO_LARGE		        =   0x1000000e
MACH_SEND_INVALID_TYPE		    =   0x1000000f
MACH_SEND_INVALID_HEADER	    =   0x10000010
MACH_SEND_INVALID_TRAILER	    =   0x10000011
MACH_SEND_INVALID_RT_OOL_SIZE	=   0x10000015
MACH_RCV_IN_PROGRESS		    =   0x10004001
MACH_RCV_INVALID_NAME		    =   0x10004002
MACH_RCV_TIMED_OUT		        =   0x10004003
MACH_RCV_TOO_LARGE		        =   0x10004004
MACH_RCV_INTERRUPTED		    =   0x10004005
MACH_RCV_PORT_CHANGED		    =   0x10004006
MACH_RCV_INVALID_NOTIFY		    =   0x10004007
MACH_RCV_INVALID_DATA		    =   0x10004008
MACH_RCV_PORT_DIED		        =   0x10004009
MACH_RCV_IN_SET			        =   0x1000400a
MACH_RCV_HEADER_ERROR		    =   0x1000400b
MACH_RCV_BODY_ERROR		        =   0x1000400c
MACH_RCV_INVALID_TYPE		    =   0x1000400d
MACH_RCV_SCATTER_SMALL		    =   0x1000400e
MACH_RCV_INVALID_TRAILER	    =   0x1000400f
MACH_RCV_IN_PROGRESS_TIMED      =   0x10004011

# CS opetions
CS_OPS_STATUS		        = 0
CS_OPS_MARKINVALID	        = 1
CS_OPS_MARKHARD		        = 2
CS_OPS_MARKKILL		        = 3
CS_OPS_CDHASH		        = 5
CS_OPS_PIDOFFSET	        = 6
CS_OPS_ENTITLEMENTS_BLOB    = 7
CS_OPS_MARKRESTRICT	        = 8
CS_OPS_SET_STATUS	        = 9
CS_OPS_BLOB		            = 10
CS_OPS_IDENTITY		        = 11
CS_OPS_CLEARINSTALLER	    = 12
CS_OPS_CLEARPLATFORM        = 13
CS_OPS_TEAMID               = 14
CS_MAX_TEAMID_LEN	        = 64

# code signing attributes of a proc
CS_VALID                    = 0x00000001
CS_ADHOC                    = 0x00000002
CS_GET_TASK_ALLOW           = 0x00000004
CS_INSTALLER                = 0x00000008
CS_FORCED_LV                = 0x00000010
CS_INVALID_ALLOWED          = 0x00000020
CS_HARD                     = 0x00000100
CS_KILL                     = 0x00000200
CS_CHECK_EXPIRATION         = 0x00000400
CS_RESTRICT                 = 0x00000800
CS_ENFORCEMENT              = 0x00001000
CS_REQUIRE_LV               = 0x00002000
CS_ENTITLEMENTS_VALIDATED   = 0x00004000
CS_NVRAM_UNRESTRICTED       = 0x00008000
CS_RUNTIME                  = 0x00010000
CS_ALLOWED_MACHO            = (CS_ADHOC | CS_HARD | CS_KILL | CS_CHECK_EXPIRATION | CS_RESTRICT | CS_ENFORCEMENT | CS_REQUIRE_LV | CS_RUNTIME)
CS_EXEC_SET_HARD            = 0x00100000
CS_EXEC_SET_KILL            = 0x00200000
CS_EXEC_SET_ENFORCEMENT     = 0x00400000
CS_EXEC_INHERIT_SIP         = 0x00800000
CS_KILLED                   = 0x01000000
CS_DYLD_PLATFORM            = 0x02000000
CS_PLATFORM_BINARY          = 0x04000000
CS_PLATFORM_PATH            = 0x08000000
CS_DEBUGGED                 = 0x10000000
CS_SIGNED                   = 0x20000000
CS_DEV_CODE                 = 0x40000000
CS_DATAVAULT_CONTROLLER     = 0x80000000
CS_ENTITLEMENT_FLAGS        = (CS_GET_TASK_ALLOW | CS_INSTALLER | CS_DATAVAULT_CONTROLLER | CS_NVRAM_UNRESTRICTED)

# executeable segment flags
CS_EXECSEG_MAIN_BINARY	    = 0x1
CS_EXECSEG_ALLOW_UNSIGNED   =0x10
CS_EXECSEG_DEBUGGER         = 0x20
CS_EXECSEG_JIT              = 0x40
CS_EXECSEG_SKIP_LV          = 0x80
CS_EXECSEG_CAN_LOAD_CDHASH  = 0x100
CS_EXECSEG_CAN_EXEC_CDHASH  = 0x200

# mach port options
MACH_MSG_OPTION_NONE        = 0x00000000
MACH_SEND_MSG               = 0x00000001
MACH_RCV_MSG                = 0x00000002
MACH_RCV_LARGE              = 0x00000004
MACH_RCV_LARGE_IDENTITY     = 0x00000008
MACH_SEND_TIMEOUT           = 0x00000010
MACH_SEND_OVERRIDE          = 0x00000020
MACH_SEND_INTERRUPT         = 0x00000040
MACH_SEND_NOTIFY            = 0x00000080
MACH_SEND_ALWAYS            = 0x00010000
MACH_SEND_TRAILER           = 0x00020000
MACH_SEND_NOIMPORTANCE      = 0x00040000
MACH_SEND_NODENAP           = MACH_SEND_NOIMPORTANCE
MACH_SEND_IMPORTANCE        = 0x00080000
MACH_SEND_SYNC_OVERRIDE     = 0x00100000
MACH_SEND_PROPAGATE_QOS     = 0x00200000
MACH_SEND_SYNC_USE_THRPRI	= MACH_SEND_PROPAGATE_QOS
MACH_SEND_KERNEL            = 0x00400000
MACH_RCV_TIMEOUT            = 0x00000100
MACH_RCV_NOTIFY             = 0x00000200
MACH_RCV_INTERRUPT          = 0x00000400
MACH_RCV_VOUCHER            = 0x00000800
MACH_RCV_OVERWRITE          = 0x00001000
MACH_RCV_SYNC_WAIT          = 0x00004000

MACH_RCV_TRAILER_NULL       = 0
MACH_RCV_TRAILER_SEQNO      = 1
MACH_RCV_TRAILER_SENDER     = 2
MACH_RCV_TRAILER_AUDIT      = 3
MACH_RCV_TRAILER_CTX        = 4
MACH_RCV_TRAILER_AV         = 7
MACH_RCV_TRAILER_LABELS     = 8
MACH_RCV_TRAILER_MASK       = (0xf << 24)
MACH_SEND_USER              = (MACH_SEND_MSG | MACH_SEND_TIMEOUT | \
                              MACH_SEND_NOTIFY | MACH_SEND_OVERRIDE | \
                              MACH_SEND_TRAILER | MACH_SEND_NOIMPORTANCE | \
                              MACH_SEND_SYNC_OVERRIDE | MACH_SEND_PROPAGATE_QOS)
MACH_RCV_USER               = (MACH_RCV_MSG | MACH_RCV_TIMEOUT | \
                              MACH_RCV_LARGE | MACH_RCV_LARGE_IDENTITY | \
                              MACH_RCV_VOUCHER | MACH_RCV_TRAILER_MASK | \
                              MACH_RCV_SYNC_WAIT)
MACH_MSG_OPTION_USER        = (MACH_SEND_USER | MACH_RCV_USER)


# inline int in error.d, not complate
EPERM           = 1
ENOENT          = 2
ESRCH           = 3
EINTR           = 4
EIO             = 5
ENXIO           = 6
E2BIG           = 7
ENOEXEC         = 8
EBADF           = 9
ECHILD          = 10
EDEADLK         = 11
ENOMEM          = 12
EACCES          = 13
EFAULT          = 14
ENOTBLK         = 15
EBUSY           = 16
EEXIST          = 17
EXDEV           = 18
ENODEV          = 19
ENOTDIR         = 20
EISDIR          = 21
EINVAL          = 22
ENFILE          = 23
EMFILE          = 24
ENOTTY          = 25
ETXTBSY         = 26
EFBIG           = 27
ENOSPC          = 28
ESPIPE          = 29
EROFS           = 30
EMLINK          = 31
EPIPE           = 32
EDOM            = 33
ERANGE          = 34
EAGAIN          = 35
EWOULDBLOCK     = 35
EINPROGRESS     = 36
EALREADY        = 37
ENOTSOCK        = 38
EDESTADDRREQ    = 39
EMSGSIZE        = 40
EPROTOTYPE      = 41
ENOPROTOOPT     = 42
EPROTONOSUPPORT = 43
ESOCKTNOSUPPORT = 44
ENOTSUP         = 45
EPFNOSUPPORT    = 46
EAFNOSUPPORT    = 47
EADDRINUSE      = 48
EADDRNOTAVAIL   = 49
ENETDOWN        = 50
ENETUNREACH     = 51
ENETRESET       = 52
ECONNABORTED    = 53
ECONNRESET      = 54
ENOBUFS         = 55
EISCONN         = 56
ENOTCONN        = 57
ESHUTDOWN       = 58
ETOOMANYREFS    = 59
ETIMEDOUT       = 60
ECONNREFUSED    = 61
ELOOP           = 62
ENAMETOOLONG    = 63
EHOSTDOWN       = 64
EHOSTUNREACH    = 65
ENOTEMPTY       = 66
EPROCLIM        = 67
EUSERS          = 68
EDQUOT          = 69
ESTALE          = 70
EREMOTE         = 71
EBADRPC         = 72
ERPCMISMATCH    = 73
EPROGUNAVAIL    = 74
EPROGMISMATCH   = 75
EPROCUNAVAIL    = 76
ENOLCK          = 77
ENOSYS          = 78
EFTYPE          = 79
EAUTH           = 80
ENEEDAUTH       = 81
EPWROFF         = 82
EDEVERR         = 83
EOVERFLOW       = 84
EBADEXEC        = 85
EBADARCH        = 86
ESHLIBVERS      = 87
EBADMACHO       = 88
ECANCELED       = 89
EIDRM           = 90
ENOMSG          = 91
EILSEQ          = 92
ENOATTR         = 93
EBADMSG         = 94
EMULTIHOP       = 95
ENODATA         = 96
ENOLINK         = 97
ENOSR           = 98
ENOSTR          = 99
EPROTO          = 100
ETIME           = 101
EOPNOTSUPP      = 102
ELAST           = 102


# shared region 
SHARED_REGION_BASE_I386	            = 0x90000000
SHARED_REGION_SIZE_I386             = 0x20000000
SHARED_REGION_NESTING_BASE_I386	    = 0x90000000
SHARED_REGION_NESTING_SIZE_I386	    = 0x20000000
SHARED_REGION_NESTING_MIN_I386      = 0x00200000
SHARED_REGION_NESTING_MAX_I386      = 0xFFE00000
SHARED_REGION_BASE_X86_64           = 0x00007FFF00000000
SHARED_REGION_SIZE_X86_64           = 0x00000000FFE00000
SHARED_REGION_NESTING_BASE_X86_64   = 0x00007FFF00000000
SHARED_REGION_NESTING_SIZE_X86_64   = 0x00000000FFE00000
SHARED_REGION_NESTING_MIN_X86_64    = 0x0000000000200000
SHARED_REGION_NESTING_MAX_X86_64    = 0xFFFFFFFFFFE00000
SHARED_REGION_BASE_PPC              = 0x90000000
SHARED_REGION_SIZE_PPC              = 0x20000000
SHARED_REGION_NESTING_BASE_PPC      = 0x90000000
SHARED_REGION_NESTING_SIZE_PPC      = 0x10000000
SHARED_REGION_NESTING_MIN_PPC       = 0x10000000
SHARED_REGION_NESTING_MAX_PPC       = 0x10000000
SHARED_REGION_BASE_PPC64            = 0x00007FFF60000000
SHARED_REGION_SIZE_PPC64            = 0x00000000A0000000
SHARED_REGION_NESTING_BASE_PPC64    = 0x00007FFF60000000
SHARED_REGION_NESTING_SIZE_PPC64    = 0x00000000A0000000
SHARED_REGION_NESTING_MIN_PPC64     = 0x0000000010000000
SHARED_REGION_NESTING_MAX_PPC64     = 0x0000000010000000
SHARED_REGION_BASE_ARM              = 0x1A000000
SHARED_REGION_SIZE_ARM              = 0x26000000
SHARED_REGION_NESTING_BASE_ARM      = 0x1A000000
SHARED_REGION_NESTING_SIZE_ARM      = 0x26000000


# fcntl 
F_DUPFD                             = 0
F_GETFD                             = 1
F_SETFD                             = 2
F_GETFL                             = 3
F_SETFL                             = 4
F_GETOWN                            = 5
F_SETOWN                            = 6
F_SETLK                             = 8
F_SETLKW                            = 9
F_SETLKWTIMEOUT                     = 10
F_DUPFD_CLOEXEC                     = 67
F_SETNOSIGPIPE                      = 73
F_GETNOSIGPIPE                      = 74
F_OFD_SETLK                         = 90
F_OFD_SETLKW                        = 91
F_OFD_SETLKWTIMEOUT                 = 93
F_SETCONFINED                       = 95
F_GETCONFINED                       = 96
F_ADDFILESIGS_RETURN                = 97

# proc info call numbers
PROC_INFO_CALL_LISTPIDS             = 0x1
PROC_INFO_CALL_PIDINFO              = 0x2
PROC_INFO_CALL_PIDFDINFO            = 0x3
PROC_INFO_CALL_KERNMSGBUF           = 0x4
PROC_INFO_CALL_SETCONTROL           = 0x5
PROC_INFO_CALL_PIDFILEPORTINFO      = 0x6
PROC_INFO_CALL_TERMINATE            = 0x7
PROC_INFO_CALL_DIRTYCONTROL         = 0x8
PROC_INFO_CALL_PIDRUSAGE            = 0x9
PROC_INFO_CALL_PIDORIGINATORINFO    = 0xa
PROC_INFO_CALL_LISTCOALITIONS       = 0xb
PROC_INFO_CALL_CANUSEFGHW           = 0xc
PROC_INFO_CALL_PIDDYNKQUEUEINFO     = 0xd
PROC_INFO_CALL_UDATA_INFO           = 0xe


PROC_PIDLISTFDS                     = 1
PROC_PIDTASKALLINFO2                = 2
PROC_PIDTBSDINFO                    = 3
PROC_PIDTASKINFO                    = 4
PROC_PIDTHREADINFO                  = 5
PROC_PIDLISTTHREADS                 = 6
PROC_PIDREGIONINFO                  = 7
PROC_PIDREGIONPATHINFO              = 8
PROC_PIDVNODEPATHINFO               = 9
PROC_PIDTHREADPATHINFO              = 10
PROC_PIDPATHINFO                    = 11
PROC_PIDWORKQUEUEINFO               = 12
PROC_PIDT_SHORTBSDINFO              = 13
PROC_PIDLISTFILEPORTS               = 14
PROC_PIDTHREADID64INFO              = 15
PROC_PID_RUSAGE                     = 16
PROC_PIDUNIQIDENTIFIERINFO          = 17
PROC_PIDT_BSDINFOWITHUNIQID         = 18
PROC_PIDARCHINFO                    = 19
PROC_PIDCOALITIONINFO               = 20
PROC_PIDNOTEEXIT                    = 21
PROC_PIDREGIONPATHINFO2             = 22
PROC_PIDREGIONPATHINFO3             = 23
PROC_PIDEXITREASONINFO              = 24
PROC_PIDEXITREASONBASICINFO         = 25
PROC_PIDLISTUPTRS                   = 26
PROC_PIDLISTDYNKQUEUES              = 27
PROC_PIDLISTTHREADIDS               = 28
PROC_PIDVMRTFAULTINFO               = 29

PROC_PIDREGIONPATHINFO_SIZE         = 1272

# syscall getattrlist
# common
ATTR_CMN_NAME                       = 0x00000001
ATTR_CMN_DEVID                      = 0x00000002
ATTR_CMN_FSID                       = 0x00000004
ATTR_CMN_OBJTYPE                    = 0x00000008
ATTR_CMN_OBJTAG                     = 0x00000010
ATTR_CMN_OBJID                      = 0x00000020
ATTR_CMN_OBJPERMANENTID             = 0x00000040
ATTR_CMN_PAROBJID                   = 0x00000080
ATTR_CMN_SCRIPT                     = 0x00000100
ATTR_CMN_CRTIME                     = 0x00000200
ATTR_CMN_MODTIME                    = 0x00000400
ATTR_CMN_CHGTIME                    = 0x00000800
ATTR_CMN_ACCTIME                    = 0x00001000
ATTR_CMN_BKUPTIME                   = 0x00002000
ATTR_CMN_FNDRINFO                   = 0x00004000
ATTR_CMN_OWNERID                    = 0x00008000
ATTR_CMN_GRPID                      = 0x00010000
ATTR_CMN_ACCESSMASK                 = 0x00020000
ATTR_CMN_FLAGS                      = 0x00040000
ATTR_CMN_GEN_COUNT                  = 0x00080000
ATTR_CMN_DOCUMENT_ID                = 0x00100000
ATTR_CMN_USERACCESS                 = 0x00200000
ATTR_CMN_EXTENDED_SECURITY          = 0x00400000
ATTR_CMN_UUID                       = 0x00800000
ATTR_CMN_GRPUUID                    = 0x01000000
ATTR_CMN_FILEID                     = 0x02000000
ATTR_CMN_PARENTID                   = 0x04000000
ATTR_CMN_FULLPATH                   = 0x08000000
ATTR_CMN_ADDEDTIME                  = 0x10000000
ATTR_CMN_ERROR                      = 0x20000000
ATTR_CMN_DATA_PROTECT_FLAGS         = 0x40000000
ATTR_CMN_RETURNED_ATTRS             = 0x80000000 
ATTR_CMN_VALIDMASK                  = 0xFFFFFFFF

# vnode type 
VNON                                = 0
VREG                                = 1
VDIR                                = 2
VBLK                                = 3
VCHR                                = 4
VLNK                                = 5
VSOCK                               = 6
VFIFO                               = 7
VBAD                                = 8
VSTR                                = 9
VCPLX                               = 10


# host info def values
HOST_BASIC_INFO                     = 1
HOST_SCHED_INFO                     = 3
HOST_RESOURCE_SIZES                 = 4
HOST_PRIORITY_INFO                  = 5
HOST_SEMAPHORE_TRAPS                = 7
HOST_MACH_MSG_TRAP                  = 8
HOST_VM_PURGABLE                    = 9
HOST_DEBUG_INFO_INTERNAL            = 10
HOST_CAN_HAS_DEBUGGER               = 11
HOST_PREFERRED_USER_ARCH            = 12


# commpage 
X8664_COMM_PAGE_START_ADDRESS       = 0x7FFFFFE00000
ARM64_COMM_PAGE_START_ADDRESS       = 0x0000000FFFFFC000

COMM_PAGE_SIGNATURE                 = 0x000   # first 16 bytes are a signature
COMM_PAGE_CPU_CAPABILITIES64        = 0x010   # uint64_t _cpu_capabilities
COMM_PAGE_UNUSED                    = 0x018   # 6 unused bytes
COMM_PAGE_VERSION                   = 0x01E   # 16-bit version
COMM_PAGE_THIS_VERSION              = 13      # in ver 13, _COMM_PAGE_NT_SHIFT defaults to 0 (was 32) 

COMM_PAGE_CPU_CAPABILITIES          = 0x020   # uint32_t _cpu_capabilities (retained for compatibility) */
COMM_PAGE_NCPUS                     = 0x022   # uint8_t number of configured CPUs (hw.logicalcpu at boot time) */
COMM_PAGE_UNUSED0                   = 0x024   # 2 unused bytes, previouly reserved for expansion of cpu_capabilities */
COMM_PAGE_CACHE_LINESIZE            = 0x026   # uint16_t cache line size */

COMM_PAGE_SCHED_GEN                 = 0x028	  # uint32_t scheduler generation number (count of pre-emptions) */
COMM_PAGE_MEMORY_PRESSURE           = 0x02c   # uint32_t copy of vm_memory_pressure */
COMM_PAGE_SPIN_COUNT                = 0x030	  # uint32_t max spin count for mutex's */

COMM_PAGE_ACTIVE_CPUS               = 0x034   # uint8_t number of active CPUs (hw.activecpu) */
COMM_PAGE_PHYSICAL_CPUS             = 0x035   # uint8_t number of physical CPUs (hw.physicalcpu_max) */
COMM_PAGE_LOGICAL_CPUS              = 0x036   # uint8_t number of logical CPUs (hw.logicalcpu_max) */
COMM_PAGE_UNUSED1                   = 0x037   # 1 unused bytes */
COMM_PAGE_MEMORY_SIZE               = 0x038   # uint64_t max memory size */

COMM_PAGE_CPUFAMILY                 = 0x040   # uint32_t hw.cpufamily, x86*/
COMM_PAGE_KDEBUG_ENABLE             = 0x044   # uint32_t export "kdebug_enable" to userspace */
COMM_PAGE_ATM_DIAGNOSTIC_CONFIG     = 0x48    # uint32_t export "atm_diagnostic_config" to userspace */

COMM_PAGE_UNUSED2                   = 0x04C   # [0x4C,0x50) unused */

COMM_PAGE_TIME_DATA_START           = 0x050   # base of offsets below (_NT_SCALE etc) */
COMM_PAGE_NT_TSC_BASE               = 0x050   # used by nanotime() */
COMM_PAGE_NT_SCALE                  = 0x058   # used by nanotime() */
COMM_PAGE_NT_SHIFT                  = 0x05c   # used by nanotime() */
COMM_PAGE_NT_NS_BASE                = 0x060   # used by nanotime() */
COMM_PAGE_NT_GENERATION             = 0x068   # used by nanotime() */
COMM_PAGE_GTOD_GENERATION           = 0x06c   # used by gettimeofday() */
COMM_PAGE_GTOD_NS_BASE              = 0x070   # used by gettimeofday() */
COMM_PAGE_GTOD_SEC_BASE             = 0x078   # used by gettimeofday() */

# APPROX_TIME must be aligned to 64-byte cache line size
COMM_PAGE_APPROX_TIME               = 0x080   # used by mach_approximate_time() */
COMM_PAGE_APPROX_TIME_SUPPORTED     = 0x088   # used by mach_approximate_time() */

# following entries to next cache line
COMM_PAGE_CONT_TIMEBASE             = 0x0C0   # used by mach_continuous_time() */
COMM_PAGE_BOOTTIME_USEC             = 0x0C8   # uint64_t boottime */

COMM_PAGE_END                       = 0xfff


#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from unicorn.x86_const import *

DWORD = 1
UINT = 1
ULONG = 1
INT = 1
BOOL = 1
SIZE_T = 1
BYTE = 1
CHAR = 1
UCHAR = 1
USHORT = 1
SHORT = 1
ULONGLONG = 2
POINTER = 3
STRING = 4
BOOLEAN = 8
BOOL = 8

def macos_kernel_api(param_num=None, params=None, passthru=False):
    def decorator(func):
        def wrapper(*args, **kwargs):
            ql = args[0]
            return ql.os.x8664_fastcall(param_num, params, func, args, kwargs, passthru)
        return wrapper
    return decorator




#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct
import time
from qiling.os.windows.const import *
from qiling.os.const import *
from qiling.os.windows.fncc import *
from qiling.os.windows.utils import *
from qiling.os.windows.thread import *
from qiling.os.windows.handle import *
from qiling.exception import *


dllname = 'kernel32_dll'

# BOOL GetModuleInformation(
#   HANDLE       hProcess,
#   HMODULE      hModule,
#   LPMODULEINFO lpmodinfo,
#   DWORD        cb
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_K32GetModuleInformation(self, address, params):
    # TODO
    return 0



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from contextlib import contextmanager

from .formats import *


# Returns subclasses recursively.
def get_all_subclasses(cls):
    all_subclasses = []

    for subclass in cls.__subclasses__():
        all_subclasses.append(subclass)
        all_subclasses.extend(get_all_subclasses(subclass))

    return all_subclasses

class CoverageFactory():
    def __init__(self):
        self.coverage_collectors = {subcls.FORMAT_NAME:subcls for subcls in get_all_subclasses(base.QlBaseCoverage)}

    @property
    def formats(self):
        return self.coverage_collectors.keys()

    def get_coverage_collector(self, ql, name):
        return self.coverage_collectors[name](ql)

factory = CoverageFactory()

@contextmanager
def collect_coverage(ql, name, coverage_file):
    """
    Context manager for emulating a given piece of code with coverage collection turned on.
    Example:
    with collect_coverage(ql, 'drcov', 'output.cov'):
        ql.run(...)
    """

    cov = factory.get_coverage_collector(ql, name)
    cov.activate()
    try:
        yield
    finally:
        cov.deactivate()
        if coverage_file:
            cov.dump_coverage(coverage_file)



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from unicorn.x86_const import *

QL_X86_F_GRANULARITY = 0x8
QL_X86_F_PROT_32 = 0x4
QL_X86_F_LONG = 0x2
QL_X86_F_AVAILABLE = 0x1

QL_X86_A_PRESENT = 0x80

QL_X86_A_PRIV_3 = 0x60
QL_X86_A_PRIV_2 = 0x40
QL_X86_A_PRIV_1 = 0x20
QL_X86_A_PRIV_0 = 0x0

QL_X86_A_CODE = 0x10
QL_X86_A_DATA = 0x10
QL_X86_A_TSS = 0x0
QL_X86_A_GATE = 0x0
QL_X86_A_EXEC = 0x8

QL_X86_A_DATA_WRITABLE = 0x2
QL_X86_A_CODE_READABLE = 0x2
QL_X86_A_DIR_CON_BIT = 0x4

QL_X86_S_GDT = 0x0
QL_X86_S_LDT = 0x4
QL_X86_S_PRIV_3 = 0x3
QL_X86_S_PRIV_2 = 0x2
QL_X86_S_PRIV_1 = 0x1
QL_X86_S_PRIV_0 = 0x0

QL_X86_GDT_ADDR = 0x30000
QL_X86_GDT_LIMIT = 0x1000
QL_X86_GDT_ENTRY_SIZE = 0x8

# These msr registers are x86 specific
FSMSR = 0xC0000100
GSMSR = 0xC0000101

# WINDOWS SETUP VALUE
# Linux also needs these
GS_SEGMENT_ADDR = 0x6000000
GS_SEGMENT_SIZE = (20 << 20)    # 20 MB

FS_SEGMENT_ADDR = 0x6000
FS_SEGMENT_SIZE = 0x6000


reg_map_8 = {
    "ah": UC_X86_REG_AH,
    "al": UC_X86_REG_AL,
    "ch": UC_X86_REG_CH,
    "cl": UC_X86_REG_CL,
    "dh": UC_X86_REG_DH,
    "dl": UC_X86_REG_DL,
    "bh": UC_X86_REG_BH,
    "bl": UC_X86_REG_BL,
}

reg_map_16 = {
    "ax": UC_X86_REG_AX,
    "cx": UC_X86_REG_CX,
    "dx": UC_X86_REG_DX,
    "bx": UC_X86_REG_BX,
    "sp": UC_X86_REG_SP,
    "bp": UC_X86_REG_BP,
    "si": UC_X86_REG_SI,
    "di": UC_X86_REG_DI,
    "ip": UC_X86_REG_IP,
}

reg_map_32 = {
    "eax": UC_X86_REG_EAX, 
    "ecx": UC_X86_REG_ECX, 
    "edx": UC_X86_REG_EDX,
    "ebx": UC_X86_REG_EBX,
    "esp": UC_X86_REG_ESP, 
    "ebp": UC_X86_REG_EBP,
    "esi": UC_X86_REG_ESI, 
    "edi": UC_X86_REG_EDI, 
    "eip": UC_X86_REG_EIP,
}

reg_map_64 = {
    "rax": UC_X86_REG_RAX,
    "rbx": UC_X86_REG_RBX, 
    "rcx": UC_X86_REG_RCX, 
    "rdx": UC_X86_REG_RDX,
    "rsi": UC_X86_REG_RSI, 
    "rdi": UC_X86_REG_RDI,
    "rbp": UC_X86_REG_RBP,
    "rsp": UC_X86_REG_RSP, 
    "r8": UC_X86_REG_R8,
    "r9": UC_X86_REG_R9, 
    "r10": UC_X86_REG_R10,
    "r11": UC_X86_REG_R11,
    "r12": UC_X86_REG_R12, 
    "r13": UC_X86_REG_R13, 
    "r14": UC_X86_REG_R14,
    "r15": UC_X86_REG_R15,
    "rip": UC_X86_REG_RIP,
}

reg_map_seg_base = {
    "fsbase" : UC_X86_REG_FS_BASE,
    "gsbase" : UC_X86_REG_GS_BASE
}

reg_map_r = {
    "r8b":  UC_X86_REG_R8B,
    "r9b":  UC_X86_REG_R9B,
    "r10b": UC_X86_REG_R10B,
    "r11b": UC_X86_REG_R11B,
    "r12b": UC_X86_REG_R12B,
    "r13b": UC_X86_REG_R13B,
    "r14b": UC_X86_REG_R14B,
    "r15b": UC_X86_REG_R15B,
    "r8w":  UC_X86_REG_R8W,
    "r9w":  UC_X86_REG_R9W,
    "r10w": UC_X86_REG_R10W,
    "r11w": UC_X86_REG_R11W,
    "r12w": UC_X86_REG_R12W,
    "r13w": UC_X86_REG_R13W,
    "r14w": UC_X86_REG_R14W,
    "r15w": UC_X86_REG_R15W,
    "r8d":  UC_X86_REG_R8D,
    "r9d":  UC_X86_REG_R9D,
    "r10d": UC_X86_REG_R10D,
    "r11d": UC_X86_REG_R11D,
    "r12d": UC_X86_REG_R12D,
    "r13d": UC_X86_REG_R13D,
    "r14d": UC_X86_REG_R14D,
    "r15d": UC_X86_REG_R15D,    
}

reg_map_cr = {
    "cr0": UC_X86_REG_CR0, 
    "cr1": UC_X86_REG_CR1,
    "cr2": UC_X86_REG_CR2, 
    "cr3": UC_X86_REG_CR3, 
    "cr4": UC_X86_REG_CR4,
    "cr5": UC_X86_REG_CR5, 
    "cr6": UC_X86_REG_CR6, 
    "cr7": UC_X86_REG_CR7,
    "cr8": UC_X86_REG_CR8,
    "cr9": UC_X86_REG_CR9,
    "cr10": UC_X86_REG_CR10,
    "cr11": UC_X86_REG_CR11,
    "cr12": UC_X86_REG_CR12,
    "cr13": UC_X86_REG_CR13,
    "cr14": UC_X86_REG_CR14,
    "cr15": UC_X86_REG_CR15,
}

reg_map_dr = {
    "dr0": UC_X86_REG_DR0, 
    "dr1": UC_X86_REG_DR1,
    "dr2": UC_X86_REG_DR2, 
    "dr3": UC_X86_REG_DR3, 
    "dr4": UC_X86_REG_DR4,
    "dr5": UC_X86_REG_DR5, 
    "dr6": UC_X86_REG_DR6, 
    "dr7": UC_X86_REG_DR7,
    "dr8": UC_X86_REG_DR8,
    "dr9": UC_X86_REG_DR9,
    "dr10": UC_X86_REG_DR10,
    "dr11": UC_X86_REG_DR11,
    "dr12": UC_X86_REG_DR12,
    "dr13": UC_X86_REG_DR13,
    "dr14": UC_X86_REG_DR14,
    "dr15": UC_X86_REG_DR15,
}

reg_map_st = {
    "st0": UC_X86_REG_ST0, 
    "st1": UC_X86_REG_ST1,
    "st2": UC_X86_REG_ST2, 
    "st3": UC_X86_REG_ST3, 
    "st4": UC_X86_REG_ST4,
    "st5": UC_X86_REG_ST5, 
    "st6": UC_X86_REG_ST6, 
    "st7": UC_X86_REG_ST7
}

reg_map_misc = {
    "ef": UC_X86_REG_EFLAGS, 
    "cs": UC_X86_REG_CS, 
    "ss": UC_X86_REG_SS,
    "ds": UC_X86_REG_DS, 
    "es": UC_X86_REG_ES, 
    "fs": UC_X86_REG_FS,
    "gs": UC_X86_REG_GS, 
}

reg_map_fp = {
    "fp0": UC_X86_REG_FP0,
    "fp1": UC_X86_REG_FP1,
    "fp2": UC_X86_REG_FP2,
    "fp3": UC_X86_REG_FP3,
    "fp4": UC_X86_REG_FP4,
    "fp5": UC_X86_REG_FP5,
    "fp6": UC_X86_REG_FP6,
    "fp7": UC_X86_REG_FP7,
}

reg_map_xmm = {
    "xmm0": UC_X86_REG_XMM0,
    "xmm1": UC_X86_REG_XMM1,
    "xmm2": UC_X86_REG_XMM2,
    "xmm3": UC_X86_REG_XMM3,
    "xmm4": UC_X86_REG_XMM4,
    "xmm5": UC_X86_REG_XMM5,
    "xmm6": UC_X86_REG_XMM6,
    "xmm7": UC_X86_REG_XMM7,
    "xmm8": UC_X86_REG_XMM8,
    "xmm9": UC_X86_REG_XMM9,
    "xmm10": UC_X86_REG_XMM10,
    "xmm11": UC_X86_REG_XMM11,
    "xmm12": UC_X86_REG_XMM12,
    "xmm13": UC_X86_REG_XMM13,
    "xmm14": UC_X86_REG_XMM14,
    "xmm15": UC_X86_REG_XMM15,
    "xmm16": UC_X86_REG_XMM16,
    "xmm17": UC_X86_REG_XMM17,
    "xmm18": UC_X86_REG_XMM18,
    "xmm19": UC_X86_REG_XMM19,
    "xmm20": UC_X86_REG_XMM20,
    "xmm21": UC_X86_REG_XMM21,
    "xmm22": UC_X86_REG_XMM22,
    "xmm23": UC_X86_REG_XMM23,
    "xmm24": UC_X86_REG_XMM24,
    "xmm25": UC_X86_REG_XMM25,
    "xmm26": UC_X86_REG_XMM26,
    "xmm27": UC_X86_REG_XMM27,
    "xmm28": UC_X86_REG_XMM28,
    "xmm29": UC_X86_REG_XMM29,
    "xmm30": UC_X86_REG_XMM30,
    "xmm31": UC_X86_REG_XMM31,
}

reg_map_ymm = {
    "ymm0": UC_X86_REG_YMM0,
    "ymm1": UC_X86_REG_YMM1,
    "ymm2": UC_X86_REG_YMM2,
    "ymm3": UC_X86_REG_YMM3,
    "ymm4": UC_X86_REG_YMM4,
    "ymm5": UC_X86_REG_YMM5,
    "ymm6": UC_X86_REG_YMM6,
    "ymm7": UC_X86_REG_YMM7,
    "ymm8": UC_X86_REG_YMM8,
    "ymm9": UC_X86_REG_YMM9,
    "ymm10": UC_X86_REG_YMM10,
    "ymm11": UC_X86_REG_YMM11,
    "ymm12": UC_X86_REG_YMM12,
    "ymm13": UC_X86_REG_YMM13,
    "ymm14": UC_X86_REG_YMM14,
    "ymm15": UC_X86_REG_YMM15,
    "ymm16": UC_X86_REG_YMM16,
    "ymm17": UC_X86_REG_YMM17,
    "ymm18": UC_X86_REG_YMM18,
    "ymm19": UC_X86_REG_YMM19,
    "ymm20": UC_X86_REG_YMM20,
    "ymm21": UC_X86_REG_YMM21,
    "ymm22": UC_X86_REG_YMM22,
    "ymm23": UC_X86_REG_YMM23,
    "ymm24": UC_X86_REG_YMM24,
    "ymm25": UC_X86_REG_YMM25,
    "ymm26": UC_X86_REG_YMM26,
    "ymm27": UC_X86_REG_YMM27,
    "ymm28": UC_X86_REG_YMM28,
    "ymm29": UC_X86_REG_YMM29,
    "ymm30": UC_X86_REG_YMM30,
    "ymm31": UC_X86_REG_YMM31,
}

reg_map_zmm = {
    "zmm0": UC_X86_REG_ZMM0,
    "zmm1": UC_X86_REG_ZMM1,
    "zmm2": UC_X86_REG_ZMM2,
    "zmm3": UC_X86_REG_ZMM3,
    "zmm4": UC_X86_REG_ZMM4,
    "zmm5": UC_X86_REG_ZMM5,
    "zmm6": UC_X86_REG_ZMM6,
    "zmm7": UC_X86_REG_ZMM7,
    "zmm8": UC_X86_REG_ZMM8,
    "zmm9": UC_X86_REG_ZMM9,
    "zmm10": UC_X86_REG_ZMM10,
    "zmm11": UC_X86_REG_ZMM11,
    "zmm12": UC_X86_REG_ZMM12,
    "zmm13": UC_X86_REG_ZMM13,
    "zmm14": UC_X86_REG_ZMM14,
    "zmm15": UC_X86_REG_ZMM15,
    "zmm16": UC_X86_REG_ZMM16,
    "zmm17": UC_X86_REG_ZMM17,
    "zmm18": UC_X86_REG_ZMM18,
    "zmm19": UC_X86_REG_ZMM19,
    "zmm20": UC_X86_REG_ZMM20,
    "zmm21": UC_X86_REG_ZMM21,
    "zmm22": UC_X86_REG_ZMM22,
    "zmm23": UC_X86_REG_ZMM23,
    "zmm24": UC_X86_REG_ZMM24,
    "zmm25": UC_X86_REG_ZMM25,
    "zmm26": UC_X86_REG_ZMM26,
    "zmm27": UC_X86_REG_ZMM27,
    "zmm28": UC_X86_REG_ZMM28,
    "zmm29": UC_X86_REG_ZMM29,
    "zmm30": UC_X86_REG_ZMM30,
    "zmm31": UC_X86_REG_ZMM31,
}




#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct
import time
from qiling.os.windows.const import *
from qiling.os.const import *
from qiling.os.windows.fncc import *
from qiling.os.windows.utils import *
from qiling.os.windows.thread import *
from qiling.os.windows.handle import *
from qiling.exception import *
from qiling.os.windows.structs import *
from qiling.const import *


dllname = 'kernel32_dll'

# void Sleep(
#  DWORD dwMilliseconds
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_Sleep(ql, address, params):
    # time.sleep(params["dwMilliseconds"] * 10**(-3))
    pass


# void EnterCriticalSection(
#  LPCRITICAL_SECTION lpCriticalSection
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_EnterCriticalSection(ql, address, params):
    return 0


# void LeaveCriticalSection(
#  LPCRITICAL_SECTION lpCriticalSection
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_LeaveCriticalSection(ql, address, params):
    return 0


# void DeleteCriticalSection(
#   LPCRITICAL_SECTION lpCriticalSection
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_DeleteCriticalSection(ql, address, params):
    return 0


# void InitializeCriticalSection(
#   LPCRITICAL_SECTION lpCriticalSection
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_InitializeCriticalSection(ql, address, params):
    return 1


# BOOL InitializeCriticalSectionEx(
#   LPCRITICAL_SECTION lpCriticalSection,
#   DWORD              dwSpinCount,
#   DWORD              Flags
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_InitializeCriticalSectionEx(ql, address, params):
    return 1


# BOOL InitializeCriticalSectionAndSpinCount(
#  LPCRITICAL_SECTION lpCriticalSection,
#  DWORD              dwSpinCount
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'DWORD': 'UINT'})
def hook_InitializeCriticalSectionAndSpinCount(ql, address, params):
    return 1


# DWORD WaitForSingleObject(
#   HANDLE hHandle,
#   DWORD  dwMilliseconds
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_WaitForSingleObject(ql, address, params):
    ret = 0
    hHandle = params["hHandle"]
    dwMilliseconds = params["dwMilliseconds"]

    handle = ql.os.handle_manager.get(hHandle)
    if handle:
        target_thread = handle.obj
        ql.os.thread_manager.cur_thread.waitfor(target_thread)

    return ret


# DWORD WaitForSingleObjectEx(
#   HANDLE hHandle,
#   DWORD  dwMilliseconds
#   BOOL   bAlertable
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_WaitForSingleObjectEx(ql, address, params):
    ret = 0
    hHandle = params["hHandle"]
    dwMilliseconds = params["dwMilliseconds"]
    alertable = params["bAlertable"]

    target_thread = ql.os.handle_manager.get(hHandle).obj
    ql.os.thread_manager.cur_thread.waitfor(target_thread)

    return ret


# DWORD WaitForMultipleObjects(
#   DWORD        nCount,
#   const HANDLE *lpHandles,
#   BOOL         bWaitAll,
#   DWORD        dwMilliseconds
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'HANDLE': 'POINTER'})
def hook_WaitForMultipleObjects(ql, address, params):
    ret = 0
    nCount = params["nCount"]
    lpHandles = params["lpHandles"]
    bWaitAll = params["bWaitAll"]
    dwMilliseconds = params["dwMilliseconds"]

    for i in range(nCount):
        handle_value = ql.unpack(ql.mem.read(lpHandles + i * ql.pointersize, ql.pointersize))
        if handle_value != 0:
            thread = ql.os.handle_manager.get(handle_value).obj
            ql.os.thread_manager.cur_thread.waitfor(thread)

    return ret


# HANDLE OpenMutexW(
#   DWORD   dwDesiredAccess,
#   BOOL    bInheritHandle,
#   LPCWSTR lpName
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_OpenMutexW(ql, address, params):
    # The name can have a "Global" or "Local" prefix to explicitly open an object in the global or session namespace.
    # It can also have no prefix
    try:
        _type, name = params["lpName"].split("\\")
    except ValueError:
        name = params["lpName"]
        _type = ""

    handle = ql.os.handle_manager.search(name)
    if _type == "Global":
        # if is global is a Windows lock. We always return a valid handle because we have no way to emulate them
        # example sample: Gandcrab e42431d37561cc695de03b85e8e99c9e31321742
        if handle is None:
            return 0xD10C
        else:
            mutex = handle.obj
            if mutex.isFree():
                mutex.lock()
            else:
                raise QlErrorNotImplemented("API not implemented")
    else:
        if handle is None:
            # If a named mutex does not exist, the function fails and GetLastError returns ERROR_FILE_NOT_FOUND.
            ql.os.last_error = ERROR_FILE_NOT_FOUND
            return 0
        else:
            raise QlErrorNotImplemented("API not implemented")


# HANDLE OpenMutexA(
#   DWORD   dwDesiredAccess,
#   BOOL    bInheritHandle,
#   LPCSTR lpName
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_OpenMutexA(ql, address, params):
    return hook_OpenMutexW.__wrapped__(ql, address, params)


# HANDLE CreateMutexW(
#   LPSECURITY_ATTRIBUTES lpMutexAttributes,
#   BOOL                  bInitialOwner,
#   LPCWSTR               lpName
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_CreateMutexW(ql, address, params):
    try:
        _type, name = params["lpName"].split("\\")
    except ValueError:
        name = params["lpName"]
        _type = ""

    owning = params["bInitialOwner"]
    handle = ql.os.handle_manager.search(name)
    if handle is not None:
        # ql.os.last_error = ERROR_ALREADY_EXISTS
        return 0
    else:
        mutex = Mutex(name, _type)
        if owning:
            mutex.lock()
        handle = Handle(obj=mutex, name=name)
        ql.os.handle_manager.append(handle)

    return handle.id


# HANDLE CreateMutexA(
#   LPSECURITY_ATTRIBUTES lpMutexAttributes,
#   BOOL                  bInitialOwner,
#   LPCSTR               lpName
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_CreateMutexA(ql, address, params):
    return hook_CreateMutexW.__wrapped__(ql, address, params)

# BOOL ReleaseMutex(
#   HANDLE hMutex
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_ReleaseMutex(ql, address, params):
    hMutex = params["hMutex"]
    handle = ql.os.handle_manager.get(hMutex)
    if not handle:
        ql.os.last_error = ERROR_INVALID_HANDLE
        return 0

    mutex = handle.obj

    if not mutex or not isinstance(mutex, Mutex):
        return 0

    if mutex.isFree():
        ql.os.last_error = ERROR_NOT_OWNER
        return 0

    # FIXME: Only the owner is allowed to do this!
    mutex.unlock()
    return 1

# HANDLE CreateEventA(
#  LPSECURITY_ATTRIBUTES lpEventAttributes,
#  BOOL                  bManualReset,
#  BOOL                  bInitialState,
#  LPCSTR                lpName
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_CreateEventA(ql, address, params):
    """
    Implementation seems similar enough to Mutex to just use it
    """
    try:
        namespace, name = params["lpName"].split("\\")
    except ValueError:
        name = params["lpName"]
        namespace = ""
    handle = ql.os.handle_manager.search(name)
    if handle is not None:
        ql.os.last_error = ERROR_ALREADY_EXISTS
    else:
        mutex = Mutex(name, namespace)
        if params['bInitialState']:
            mutex.lock()
        handle = Handle(obj=mutex, name=name)
        ql.os.handle_manager.append(handle)
    return handle.ID


# HANDLE CreateEventW(
#  LPSECURITY_ATTRIBUTES lpEventAttributes,
#  BOOL                  bManualReset,
#  BOOL                  bInitialState,
#  LPCWSTR               lpName
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_CreateEventW(ql, address, params):
    return hook_CreateEventA.__wrapped__(ql, address, params)


# void InitializeSRWLock(
#  PSRWLOCK SRWLock
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_InitializeSRWLock(ql, address, params):
    return


# void AcquireSRWLockExclusive(
#   PSRWLOCK SRWLock
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_AcquireSRWLockExclusive(ql, address, params):
    return


# void AcquireSRWLockShared(
#   PSRWLOCK SRWLock
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_AcquireSRWLockShared(ql, address, params):
    return


# void ReleaseSRWLockExclusive(
#   PSRWLOCK SRWLock
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_ReleaseSRWLockExclusive(ql, address, params):
    return


# void ReleaseSRWLockShared(
#   PSRWLOCK SRWLock
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_ReleaseSRWLockShared(ql, address, params):
    return



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

def dxeapi(param_num=None, params=None):
    def decorator(func):
        def wrapper(*args, **kwargs):
            ql = args[0]
            arg = (ql, ql.reg.arch_pc, {})

            f = ql.loader.user_defined_api.get(func.__name__, func)

            ql.os.winapi_func_onenter = ql.loader.user_defined_api_onenter.get(func.__name__, None)
            ql.os.winapi_func_onexit = ql.loader.user_defined_api_onexit.get(func.__name__, None)

            return ql.os.x8664_fastcall(param_num, params, f, arg, kwargs)

        return wrapper

    return decorator



#!/usr/bin/env python3
#
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import struct
import time
from qiling.os.windows.const import *
from qiling.os.const import *
from qiling.os.windows.fncc import *
from qiling.os.windows.utils import *
from qiling.os.windows.thread import *
from qiling.os.windows.handle import *
from qiling.exception import *


dllname = 'kernel32_dll'

# BOOL DeviceIoControl(
#   HANDLE       hDevice,
#   DWORD        dwIoControlCode,
#   LPVOID       lpInBuffer,
#   DWORD        nInBufferSize,
#   LPVOID       lpOutBuffer,
#   DWORD        nOutBufferSize,
#   LPDWORD      lpBytesReturned,
#   LPOVERLAPPED lpOverlapped
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_DeviceIoControl(ql, address, params):
    operation = params["dwIoControlCode"]
    data = params["lpInBuffer"]
    output = params["lpOutBuffer"]
    output_size = params["lpBytesReturned"]
    # TODO implement operations. Did not find controlCodes values
    return 1



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#
import struct

from qiling.os.windows.fncc import *
from qiling.os.const import *
from qiling.os.windows.utils import *
from qiling.os.windows.handle import *
from qiling.os.windows.const import *
from qiling.os.windows.structs import *

dllname = 'advapi32_dll'

def _RegOpenKey(ql, address, params):
    hKey = params["hKey"]
    s_lpSubKey = params["lpSubKey"]
    phkResult = params["phkResult"]
    ql.log.debug("Key %s %s" % (hKey, s_lpSubKey))

    if hKey not in REG_KEYS:
        ql.log.debug("Key %s %s not present" % (hKey, s_lpSubKey))
        return ERROR_FILE_NOT_FOUND
    else:
        s_hKey = REG_KEYS[hKey]
    key = s_hKey + "\\" + s_lpSubKey

    # Keys in the profile are saved as KEY\PARAM = VALUE, so i just want to check that the key is the same
    keys_profile = [key.rsplit("\\", 1)[0] for key in ql.os.profile["REGISTRY"].keys()]
    if key.lower() in keys_profile:
        ql.log.debug("Using profile for key of  %s" % key)
        ql.os.registry_manager.access(key)
    else:
        if not ql.os.registry_manager.exists(key):
            ql.log.debug("Value key %s not present" % key)
            return ERROR_FILE_NOT_FOUND

    # new handle
    new_handle = Handle(obj=key)
    ql.os.handle_manager.append(new_handle)
    if phkResult != 0:
        ql.mem.write(phkResult, ql.pack(new_handle.id))
    return ERROR_SUCCESS


def RegQueryValue(ql, address, params):
    ret = ERROR_SUCCESS

    hKey = params["hKey"]
    s_lpValueName = params["lpValueName"]
    lpType = params["lpType"]
    lpData = params["lpData"]
    lpcbData = params["lpcbData"]
    s_hKey = ql.os.handle_manager.get(hKey).obj
    params["hKey"] = s_hKey
    # read reg_type
    if lpType != 0:
        reg_type = ql.unpack32(ql.mem.read(lpType, 4))
    else:
        reg_type = Registry.RegNone
    try:
        # Keys in the profile are saved as KEY\PARAM = VALUE, so i just want to check that the key is the same
        value = ql.os.profile["REGISTRY"][s_hKey + "\\" + s_lpValueName]
        ql.log.debug("Using profile for value of key %s" % (s_hKey + "\\" + s_lpValueName,))
        # TODO i have no fucking idea on how to set a None value, fucking configparser
        if value == "None":
            return ERROR_FILE_NOT_FOUND
        reg_type = 0x0001
        # set that the registry has been accessed
        ql.os.registry_manager.access(s_hKey, s_lpValueName, value, reg_type)

    except KeyError:
        # Read the registry
        reg_type, value = ql.os.registry_manager.read(s_hKey, s_lpValueName, reg_type)

    # error key
    if reg_type is None or value is None:
        ql.log.debug("Key value not found")
        return ERROR_FILE_NOT_FOUND
    else:
        # set lpData
        length = ql.os.registry_manager.write_reg_value_into_mem(value, reg_type, lpData)
        # set lpcbData
        max_size = int.from_bytes(ql.mem.read(lpcbData, 4), byteorder="little")
        ql.mem.write(lpcbData, ql.pack(length))
        if max_size < length:
            ret = ERROR_MORE_DATA

    return ret


# LSTATUS RegOpenKeyExA(
#   HKEY   hKey,
#   LPCSTR lpSubKey,
#   DWORD  ulOptions,
#   REGSAM samDesired,
#   PHKEY  phkResult
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegOpenKeyExA(ql, address, params):
    return _RegOpenKey(ql, address, params)


# LSTATUS RegOpenKeyExW(
#   HKEY    hKey,
#   LPCWSTR lpSubKey,
#   DWORD   ulOptions,
#   REGSAM  samDesired,
#   PHKEY   phkResult
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegOpenKeyExW(ql, address, params):
    return _RegOpenKey(ql, address, params)


# LSTATUS RegOpenKeyW(
#   HKEY    hKey,
#   LPCWSTR lpSubKey,
#   PHKEY   phkResult
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegOpenKeyW(ql, address, params):
    return _RegOpenKey(ql, address, params)


# LSTATUS RegOpenKeyA(
#   HKEY    hKey,
#   LPCSTR lpSubKey,
#   PHKEY   phkResult
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegOpenKeyA(ql, address, params):
    return _RegOpenKey(ql, address, params)


# LSTATUS RegQueryValueExA(
#   HKEY    hKey,
#   LPCSTR  lpValueName,
#   LPDWORD lpReserved,
#   LPDWORD lpType,
#   LPBYTE  lpData,
#   LPDWORD lpcbData
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegQueryValueExA(ql, address, params):
    return RegQueryValue(ql, address, params)


# LSTATUS RegQueryValueExW(
#   HKEY    hKey,
#   LPCWSTR lpValueName,
#   LPDWORD lpReserved,
#   LPDWORD lpType,
#   LPBYTE  lpData,
#   LPDWORD lpcbData
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegQueryValueExW(ql, address, params):
    return RegQueryValue(ql, address, params)


# LSTATUS RegCloseKey(
#   HKEY hKey
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegCloseKey(ql, address, params):
    ret = ERROR_SUCCESS
    hKey = params["hKey"]
    ql.os.handle_manager.delete(hKey)
    return ret


# LSTATUS RegCreateKeyA(
#   HKEY   hKey,
#   LPCSTR lpSubKey,
#   PHKEY  phkResult
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegCreateKeyA(ql, address, params):
    return hook_RegCreateKeyW.__wrapped__(ql, address, params)


# LSTATUS RegCreateKeyW(
#   HKEY   hKey,
#   LPCWSTR lpSubKey,
#   PHKEY  phkResult
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegCreateKeyW(ql, address, params):
    ret = ERROR_SUCCESS

    hKey = params["hKey"]
    s_lpSubKey = params["lpSubKey"]
    phkResult = params["phkResult"]

    if not (hKey in REG_KEYS):
        return 2
    else:
        s_hKey = REG_KEYS[hKey]
        params["hKey"] = s_hKey
        if not ql.os.registry_manager.exists(s_hKey + "\\" + s_lpSubKey):
            ret = ERROR_SUCCESS
            ql.os.registry_manager.create(s_hKey + "\\" + s_lpSubKey)

    # new handle
    if ret == ERROR_SUCCESS:
        new_handle = Handle(obj=s_hKey + "\\" + s_lpSubKey)
        ql.os.handle_manager.append(new_handle)
        if phkResult != 0:
            ql.mem.write(phkResult, ql.pack(new_handle.id))
    else:
        new_handle = 0

    return ret


# LSTATUS RegCreateKeyExW(
#   HKEY                        hKey,
#   LPCWSTR                     lpSubKey,
#   DWORD                       Reserved,
#   LPWSTR                      lpClass,
#   DWORD                       dwOptions,
#   REGSAM                      samDesired,
#   const LPSECURITY_ATTRIBUTES lpSecurityAttributes,
#   PHKEY                       phkResult,
#   LPDWORD                     lpdwDisposition
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'DWORD': 'POINTER'})
def hook_RegCreateKeyExW(ql, address, params):
    ret = ERROR_SUCCESS

    hKey = params["hKey"]
    s_lpSubKey = params["lpSubKey"]
    phkResult = params["phkResult"]

    if not (hKey in REG_KEYS):
        return 2
    else:
        s_hKey = REG_KEYS[hKey]
        params["hKey"] = s_hKey
        if not ql.os.registry_manager.exists(s_hKey + "\\" + s_lpSubKey):
            ret = ERROR_SUCCESS
            ql.os.registry_manager.create(s_hKey + "\\" + s_lpSubKey)

    # new handle
    if ret == ERROR_SUCCESS:
        new_handle = Handle(obj=s_hKey + "\\" + s_lpSubKey)
        ql.os.handle_manager.append(new_handle)
        if phkResult != 0:
            ql.mem.write(phkResult, ql.pack(new_handle.id))
    else:
        new_handle = 0

    return ret


# LSTATUS RegSetValueA(
#   HKEY   hKey,
#   LPCSTR lpSubKey,
#   DWORD  dwType,
#   LPCSTR lpData,
#   DWORD  cbData
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegSetValueA(ql, address, params):
    ret = ERROR_SUCCESS

    hKey = params["hKey"]
    s_lpSubKey = params["lpSubKey"]
    dwType = params["dwType"]
    s_lpData = params["lpData"]
    cbData = params["cbData"]

    s_hKey = ql.os.handle_manager.get(hKey).obj
    # this is done so the print_function would print the correct value
    params["hKey"] = s_hKey

    ql.os.registry_manager.write(s_hKey, s_lpSubKey, dwType, s_lpData)

    return ret

# LSTATUS RegSetValueExA(
#   HKEY       hKey,
#   LPCSTR     lpValueName,
#   DWORD      Reserved,
#   DWORD      dwType,
#   const BYTE *lpData,
#   DWORD      cbData
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegSetValueExA(ql, address, params):
    ret = ERROR_SUCCESS

    hKey = params["hKey"]
    s_lpValueName = params["lpValueName"]
    dwType = params["dwType"]
    s_lpData = params["lpData"]
    cbData = params["cbData"]

    s_hKey = ql.os.handle_manager.get(hKey).obj
    params["hKey"] = s_hKey

    ql.os.registry_manager.write(s_hKey, s_lpValueName, dwType, s_lpData)

    return ret



# LSTATUS RegSetValueExW(
#   HKEY       hKey,
#   LPCWSTR    lpValueName,
#   DWORD      Reserved,
#   DWORD      dwType,
#   const BYTE *lpData,
#   DWORD      cbData
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegSetValueExW(ql, address, params):
    ret = ERROR_SUCCESS

    hKey = params["hKey"]
    s_lpValueName = params["lpValueName"]
    dwType = params["dwType"]
    s_lpData = params["lpData"]
    cbData = params["cbData"]

    s_hKey = ql.os.handle_manager.get(hKey).obj
    # this is done so the print_function would print the correct value
    params["hKey"] = s_hKey

    ql.os.registry_manager.write(s_hKey, s_lpValueName, dwType, s_lpData)

    return ret


# LSTATUS RegDeleteKeyA(
#   HKEY   hKey,
#   LPCSTR lpSubKey
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegDeleteKeyA(ql, address, params):
    return hook_RegDeleteKeyW.__wrapped__(ql, address, params)


# LSTATUS RegDeleteKeyW(
#   HKEY   hKey,
#   LPCWSTR lpSubKey
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegDeleteKeyW(ql, address, params):
    ret = ERROR_SUCCESS

    hKey = params["hKey"]
    s_lpSubKey = params["lpSubKey"]

    s_hKey = ql.os.handle_manager.get(hKey).obj
    params["hKey"] = s_hKey

    ql.os.registry_manager.delete(s_hKey, s_lpSubKey)

    return ret


# LSTATUS RegDeleteValueA(
#   HKEY    hKey,
#   LPCSTR lpValueName
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegDeleteValueA(ql, address, params):
    return hook_RegDeleteValueW.__wrapped__(ql, address, params)


# LSTATUS RegDeleteValueW(
#   HKEY    hKey,
#   LPCWSTR lpValueName
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegDeleteValueW(ql, address, params):
    ret = ERROR_SUCCESS

    hKey = params["hKey"]
    s_lpValueName = params["lpValueName"]

    s_hKey = ql.os.handle_manager.get(hKey).obj
    params["hKey"] = s_hKey

    ql.os.registry_manager.delete(s_hKey, s_lpValueName)

    return ret


# BOOL GetTokenInformation(
#   HANDLE                  TokenHandle,
#   TOKEN_INFORMATION_CLASS TokenInformationClass,
#   LPVOID                  TokenInformation,
#   DWORD                   TokenInformationLength,
#   PDWORD                  ReturnLength
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetTokenInformation(ql, address, params):
    id_token = params["TokenHandle"]
    information = params["TokenInformationClass"]
    max_size = params["TokenInformationLength"]
    return_point = params["ReturnLength"]
    dst = params["TokenInformation"]
    token = ql.os.handle_manager.get(id_token).obj
    information_value = token.get(information)
    ql.mem.write(return_point, len(information_value).to_bytes(4, byteorder="little"))
    return_size = int.from_bytes(ql.mem.read(return_point, 4), byteorder="little")
    ql.log.debug("The target is checking for its permissions")
    if return_size > max_size:
        ql.os.last_error = ERROR_INSUFFICIENT_BUFFER
        return 0
    if dst != 0:
        ql.mem.write(dst, information_value)
        return 1
    else:
        raise QlErrorNotImplemented("API not implemented")


# PUCHAR GetSidSubAuthorityCount(
#   PSID pSid
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_GetSidSubAuthorityCount(ql, address, params):
    sid = ql.os.handle_manager.get(params["pSid"]).obj
    addr_authority_count = sid.addr + 1  # +1 because the first byte is revision
    return addr_authority_count


# PDWORD GetSidSubAuthority(
#   PSID  pSid,
#   DWORD nSubAuthority
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params_type={'DWORD': 'INT'})
def hook_GetSidSubAuthority(ql, address, params):
    num = params["nSubAuthority"]
    sid = ql.os.handle_manager.get(params["pSid"]).obj
    addr_authority = sid.addr + 8 + (ql.pointersize * num)
    return addr_authority

# LSTATUS RegEnumValueA(
#   HKEY    hKey,
#   DWORD   dwIndex,
#   LPSTR   lpValueName,
#   LPDWORD lpcchValueName,
#   LPDWORD lpReserved,
#   LPDWORD lpType,
#   LPBYTE  lpData,
#   LPDWORD lpcbData
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_RegEnumValueA(ql, address, params):
    return 259 # ERROR_NO_MORE_ITEMS

# SC_HANDLE OpenSCManagerA(
#   LPCSTR lpMachineName,
#   LPCSTR lpDatabaseName,
#   DWORD  dwDesiredAccess
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_OpenSCManagerA(ql, address, params):
    lpMachineName = params["lpMachineName"]
    lpDatabaseName = params["lpDatabaseName"]
    sc_handle_name = "sc_%s_%s" % (lpMachineName, lpDatabaseName)
    new_handle = ql.os.handle_manager.search(sc_handle_name)
    if new_handle is None:
        new_handle = Handle(name=sc_handle_name)
        ql.os.handle_manager.append(new_handle)
    return new_handle.id

# SC_HANDLE CreateServiceA(
#   SC_HANDLE hSCManager,
#   LPCSTR    lpServiceName,
#   LPCSTR    lpDisplayName,
#   DWORD     dwDesiredAccess,
#   DWORD     dwServiceType,
#   DWORD     dwStartType,
#   DWORD     dwErrorControl,
#   LPCSTR    lpBinaryPathName,
#   LPCSTR    lpLoadOrderGroup,
#   LPDWORD   lpdwTagId,
#   LPCSTR    lpDependencies,
#   LPCSTR    lpServiceStartName,
#   LPCSTR    lpPassword
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={
    "hSCManager":HANDLE,
    "lpServiceName": STRING,
    "lpDisplayName": STRING,
    "dwDesiredAccess": DWORD,
    "dwServiceType": DWORD,
    "dwStartType": DWORD,
    "dwErrorControl": DWORD,
    "lpBinaryPathName": STRING,
    "lpLoadOrderGroup": STRING,
    "lpdwTagId": POINTER,
    "lpDependencies": STRING,
    "lpServiceStartName": STRING,
    "lpPassword": STRING
    })
def hook_CreateServiceA(ql, address, params):
    hSCManager = params["hSCManager"]
    lpServiceName = params["lpServiceName"]
    lpBinaryPathName = params["lpBinaryPathName"]
    ql.os.services[lpServiceName] = lpBinaryPathName
    new_handle = Handle(obj=hSCManager, name=lpServiceName)
    ql.os.handle_manager.append(new_handle)
    return new_handle.id

# SC_HANDLE OpenServiceA(
#   SC_HANDLE hSCManager,
#   LPCSTR    lpServiceName,
#   DWORD     dwDesiredAccess
# );
@winsdkapi(cc=STDCALL, dllname=dllname,replace_params={
    "hSCManager":HANDLE,
    "lpServiceName": STRING,
    "dwDesiredAccess": DWORD    
})
def hook_OpenServiceA(ql, address, params):
    hSCManager = params["hSCManager"]
    lpServiceName = params["lpServiceName"]
    if lpServiceName in ql.os.services:
        new_handle = Handle(obj=hSCManager, name=lpServiceName)
        ql.os.handle_manager.append(new_handle)
        return new_handle.id
    else:
        return 0

# BOOL CloseServiceHandle(
#   SC_HANDLE hSCObject
# );
@winsdkapi(cc=STDCALL, dllname=dllname, replace_params={"hSCObject":HANDLE})
def hook_CloseServiceHandle(ql, address, params):
    hSCObject = params["hSCObject"]
    ql.os.handle_manager.delete(hSCObject)
    return 1

# BOOL StartServiceA(
#   SC_HANDLE hService,
#   DWORD     dwNumServiceArgs,
#   LPCSTR    *lpServiceArgVectors
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_StartServiceA(ql, address, params):
    return 1

# BOOL AllocateAndInitializeSid(
#   PSID_IDENTIFIER_AUTHORITY pIdentifierAuthority,
#   BYTE                      nSubAuthorityCount,
#   DWORD                     nSubAuthority0,
#   DWORD                     nSubAuthority1,
#   DWORD                     nSubAuthority2,
#   DWORD                     nSubAuthority3,
#   DWORD                     nSubAuthority4,
#   DWORD                     nSubAuthority5,
#   DWORD                     nSubAuthority6,
#   DWORD                     nSubAuthority7,
#   PSID                      *pSid
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_AllocateAndInitializeSid(ql, address, params):
    count = params["nSubAuthorityCount"]
    subs = b""
    for i in range(count):
        sub = params["nSubAuthority" + str(i)]
        subs += sub.to_bytes(4, "little")
    sid = Sid(ql, revision=1, identifier=5, subs=subs, subs_count=count)
    sid_addr = ql.os.heap.alloc(sid.size)
    sid.write(sid_addr)
    handle = Handle(obj=sid, id=sid_addr)
    ql.os.handle_manager.append(handle)
    dest = params["pSid"]
    ql.mem.write(dest, ql.pack(sid_addr))
    return 1


# Some default Sids:
__adminsid = None # Administrators (S-1-5-32-544)
__userssid = None # All Users (S-1-5-32-545)
__guestssid = None # All Users (S-1-5-32-546)
__poweruserssid = None # Power Users (S-1-5-32-547)


def get_adminsid(ql):
    global __adminsid
    if __adminsid == None:
        # nSubAuthority0 = SECURITY_BUILTIN_DOMAIN_RID[0x20], nSubAuthority1 = DOMAIN_ALIAS_RID_ADMINS[0x220]
        subs = b"\x20\x00\x00\x00\x20\x02\x00\x00"
        __adminsid = Sid(ql, revision=1, identifier=5, subs=subs, subs_count=2)
    return __adminsid

def get_userssid(ql):
    global __userssid
    if __userssid == None:
        # nSubAuthority0 = SECURITY_BUILTIN_DOMAIN_RID[0x20], nSubAuthority1 = DOMAIN_ALIAS_RID_USERS[0x221]
        subs = b"\x20\x00\x00\x00\x21\x02\x00\x00"
        __userssid = Sid(ql, revision=1, identifier=5, subs=subs, subs_count=2)
    return __userssid

def get_guestssid(ql):
    global __guestssid
    if __guestssid == None:
        # nSubAuthority0 = SECURITY_BUILTIN_DOMAIN_RID[0x20], nSubAuthority1 = DOMAIN_ALIAS_RID_GUESTS[0x222]
        subs = b"\x20\x00\x00\x00\x22\x02\x00\x00"
        __guestssid = Sid(ql, revision=1, identifier=5, subs=subs, subs_count=2)
    return __guestssid

def get_poweruserssid(ql):
    global __poweruserssid
    if __poweruserssid == None:
        # nSubAuthority0 = SECURITY_BUILTIN_DOMAIN_RID[0x20], nSubAuthority1 = DOMAIN_ALIAS_RID_POWER_USERS[0x223]
        subs = b"\x20\x00\x00\x00\x23\x02\x00\x00"
        __poweruserssid = Sid(ql, revision=1, identifier=5, subs=subs, subs_count=2)
    return __poweruserssid


# BOOL WINAPI CheckTokenMembership(IN HANDLE TokenHandle,
#   IN PSID SidToCheck,
#   OUT PBOOL IsMember
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_CheckTokenMembership(ql, address, params):
    token_handle = params["TokenHandle"]
    sid = ql.os.handle_manager.get(params["SidToCheck"]).obj
    # If TokenHandle is NULL, CheckTokenMembership uses the impersonation token of the calling thread.
    IsMember = 0
    if token_handle == 0:
        # For now, treat power users as admins
        if get_adminsid(ql) == sid or get_poweruserssid(ql) == sid:
            IsMember = 1 if ql.os.profile["SYSTEM"]["permission"] == "root" else 0
        elif get_userssid(ql) == sid:
            # FIXME: is this true for all tokens? probably not...
            IsMember = 1
        elif get_guestssid(ql) == sid:
            IsMember = 0
        else:
            assert False, 'unimplemented'
    else:
        assert False, 'unimplemented'
    ql.mem.write(params['IsMember'], ql.pack(IsMember))
    return 1


# PVOID FreeSid(
#   PSID pSid
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_FreeSid(ql, address, params):
    ql.os.heap.free(params["pSid"])
    return 0


# BOOL EqualSid(
#   PSID pSid1,
#   PSID pSid2
# );
@winsdkapi(cc=STDCALL, dllname=dllname)
def hook_EqualSid(ql, address, params):
    # TODO once i have understood better how SID are wrote in memory. Fucking documentation
    # technically this one should be my SID that i created at the start. I said should, because when testing, it has a
    # different address. Why? No idea
    # sid1 = ql.os.handle_manager.get(params["pSid1"]).obj
    sid2 = ql.os.handle_manager.get(params["pSid2"]).obj
    # return sid1 == sid2
    return 0



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

# reference to <Mac OS X and IOS Internals: To the Apple's Core>

from struct import pack, unpack

from qiling.const import *

# define in kernel osfmk/mach/message.h
# mach_msg_header_t:
#   mach_msg_bits_t	msgh_bits;                  unsigned int 
#   mach_msg_size_t	msgh_size;                  4 bytes
#   mach_port_t		msgh_remote_port;           4 bytes
#   mach_port_t		msgh_local_port;            4 bytes
#   mach_port_name_t	msgh_voucher_port;      4 bytes
#   mach_msg_id_t		msgh_id;                4 bytes
class MachMsgHeader():

    def __init__(self, ql):
        self.header_size = 24
        self.ql = ql
        self.msgh_bits = None
        self.msgh_size = None
        self.msgh_remote_port = None
        self.msgh_local_port = None
        self.msgh_voucher_port = None
        self.msgh_id = None
    
    def read_header_from_mem(self, addr):
        self.msgh_bits = unpack("<L", self.ql.mem.read(addr, 0x4))[0]
        self.msgh_size = unpack("<L", self.ql.mem.read(addr + 0x4, 0x4))[0]
        self.msgh_remote_port = unpack("<L", self.ql.mem.read(addr + 0x8, 0x4))[0]
        self.msgh_local_port = unpack("<L", self.ql.mem.read(addr + 0xc, 0x4))[0]
        self.msgh_voucher_port = unpack("<L", self.ql.mem.read(addr + 0x10, 0x4))[0]
        self.msgh_id = unpack("<L", self.ql.mem.read(addr + 0x14, 0x4))[0]
        # print("size !!!!! {}".format(self.msgh_size))

    # def __str__(self):
    #     return "[MachMsg] bits :{}, size:{}, remote port:{}, local port:{}, voucher port:{}, id:{}".format(
    #         self.msgh_bits,
    #         self.msgh_size,
    #         self.msgh_remote_port,
    #         self.msgh_local_port,
    #         self.msgh_voucher_port,
    #         self.msgh_id,
    #     )


# Mach message Class 
# mach msg: header + content + trailer
class MachMsg():
    def __init__(self, ql):
        self.ql = ql
        self.header = MachMsgHeader(self.ql)
        self.content = b''
        self.trailer = b''
        pass
    
    def read_msg_from_mem(self, addr, size):
        self.header = self.read_msg_header(addr, size)
        # between header and content is 4 byte \x00
        self.content = self.read_msg_content(addr + self.header.header_size, size - self.header.header_size)

    def write_msg_to_mem(self, addr):
        self.ql.mem.write(addr, pack("<L", self.header.msgh_bits))
        self.ql.mem.write(addr + 0x4, pack("<L", self.header.msgh_size))
        self.ql.mem.write(addr + 0x8, pack("<L", self.header.msgh_remote_port))
        self.ql.mem.write(addr + 0xc, pack("<L", self.header.msgh_local_port))
        self.ql.mem.write(addr + 0x10, pack("<L", self.header.msgh_voucher_port))
        self.ql.mem.write(addr + 0x14, pack("<L", self.header.msgh_id))
        if self.content:
            self.ql.mem.write(addr + 0x18, self.content)
        if self.trailer:
            self.ql.mem.write(addr + 0x18 + len(self.content), self.trailer)

    def read_msg_header(self, addr, size):
        header = MachMsgHeader(self.ql)
        header.read_header_from_mem(addr)
        header.msgh_size = size
        return header

    def read_msg_content(self, addr, size):
        self.ql.log.debug("0x{:X}, {}".format(addr, size))
        return self.ql.mem.read(addr, size)


# Mach Port Class 
# not Finished
class MachPort():

    def __init__(self, port_name):
        self.name = port_name
        pass


# Mach Port Manager : 
#   1. handle mach msg
#   2. register some Host Port

class MachPortManager():

    def __init__(self, ql, my_port):
        self.ql = ql
        self.host_port = MachPort(0x303)
        self.clock_port = MachPort(0x803)
        self.semaphore_port = MachPort(0x903)
        self.special_port = MachPort(0x707)
        self.my_port = my_port

    def deal_with_msg(self, msg, addr):

        if msg.header.msgh_id == 200:
            # host info
            out_msg = self.ql.os.macho_host_server.host_info(msg.header, msg.content)
            out_msg.write_msg_to_mem(addr)
        elif msg.header.msgh_id == 206:
            out_msg = self.ql.os.macho_host_server.host_get_clock_service(msg.header, msg.content)
            out_msg.write_msg_to_mem(addr)
        elif msg.header.msgh_id == 3418:
            out_msg = self.ql.os.macho_task_server.semaphore_create(msg.header, msg.content)
            out_msg.write_msg_to_mem(addr)
        elif msg.header.msgh_id == 3409:
            out_msg = self.ql.os.macho_task_server.get_special_port(msg.header, msg.content)
            out_msg.write_msg_to_mem(addr)
        else:
            self.ql.log.info("Error Mach Msgid {} can not handled".format(msg.header.msgh_id))
            raise Exception("Mach Msgid Not Found")

        self.ql.log.debug("Reply-> Header: {}, Content: {}".format(out_msg.header, out_msg.content))

    def get_thread_port(self, MachoThread):
        return MachoThread.port.name

# XNU define struct :
# struct mach_msg_overwrite_trap_args {
# 	PAD_ARG_(user_addr_t, msg);                     addr length
# 	PAD_ARG_(mach_msg_option_t, option);            int
# 	PAD_ARG_(mach_msg_size_t, send_size);           unsigned int
# 	PAD_ARG_(mach_msg_size_t, rcv_size);            unsigned int
# 	PAD_ARG_(mach_port_name_t, rcv_name);           unsigned int 
# 	PAD_ARG_(mach_msg_timeout_t, timeout);          unsigned int
# 	PAD_ARG_(mach_msg_priority_t, override);        unsigned int
# 	PAD_ARG_8
# 	PAD_ARG_(user_addr_t, rcv_msg);  /* Unused on mach_msg_trap */  addr length
# };



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

import json, os, struct 

from typing import Callable, Sequence, Mapping, MutableMapping, Any
from functools import wraps

from qiling.os.const import *
from qiling.os.windows.utils import *
from qiling.const import *
from qiling.exception import *
from qiling.os.windows.structs import *
from qiling.os.os import QlOs

from .utils import *
     

def replacetype(type, specialtype=None):
    if specialtype is None:
        specialtype = {}

    if type in reptypedict.keys():
        if type not in specialtype.keys():
            return reptypedict[type]
        else:
            return specialtype[type]
    else:
        return type


# x86/x8664 PE should share Windows APIs
def winsdkapi(cc, param_num=None, dllname=None, replace_params_type=None, replace_params={}, passthru=False):
    """
    @cc: windows api calling convention, only x86 needs this, x64 is always fastcall
    @param_num: the number of function params, used by variadic functions, e.g printf
    @dllname: the name of function
    @replace_params_type: customize replace type, e.g specialtype={'int':'UINT'} means repalce 'int' to 'UINT'
    @replace_params: customize replace param_name's type, e.g specialtypeEx={'time':'int'} means
                replace the original type of time to int
    """
    def decorator(func):
        @wraps(func)
        def wrapper(*args, **kwargs):
            funcname = func.__name__[5:]
            params = {}
            funclist = []
            ql = args[0]

            if dllname is not None:
                windows_abspath = os.path.dirname(os.path.abspath(__file__))
                winsdk_path = os.path.join(windows_abspath[:-11], 'extensions', 'windows_sdk', 'defs', dllname + '.json')

                if os.path.exists(winsdk_path):
                    with open(winsdk_path, 'r') as f:
                        funclist = json.load(f)
                else:
                    ql.log.info('%s not found', winsdk_path)
                if funcname not in funclist:
                    params = replace_params
                else:
                    paramlist = funclist[funcname]

                    if len(replace_params.keys()) == len(paramlist):
                        params = replace_params
                        for key in params:
                            if isinstance(params[key], str):
                                type = replacetype(params[key], replace_params_type)
                                params[key] = eval(type)
                    else:
                        for para in paramlist:
                            name = list(para.values())[0]
                            if name == 'VOID' or (name in replace_params.keys() and replace_params[name] == ''):
                                params = {}
                                break
                            elif replace_params is not None and name in replace_params.keys():
                                type = replace_params[name]
                                params[name] = type
                            else:
                                type = list(para.values())[1]
                                if isinstance(type, dict):
                                    type = replacetype(type['name'], replace_params_type)
                                else:
                                    type = replacetype(list(para.values())[1], replace_params_type)
                            if isinstance(type, str):
                                params[name] = eval(type)
            else:
                params = replace_params

            if ql.archtype == QL_ARCH.X86:
                if cc == STDCALL:
                    return ql.os.x86_stdcall(param_num, params, func, args, kwargs, passthru)
                elif cc == CDECL:
                    return ql.os.x86_cdecl(param_num, params, func, args, kwargs, passthru)
            elif ql.archtype == QL_ARCH.X8664:
                return ql.os.x8664_fastcall(param_num, params, func, args, kwargs, passthru)
            else:
                raise QlErrorArch("Unknown self.ql.arch")
        return wrapper

    return decorator



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from qiling.os.uefi.utils import GetEfiConfigurationTable
from qiling.os.uefi.UefiBaseType import STRUCT, EFI_GUID, UINT32, UINT16
from qiling.os.uefi.UefiSpec import EFI_CONFIGURATION_TABLE

EFI_HOB_TYPE_HANDOFF		 = 0x0001
EFI_HOB_TYPE_GUID_EXTENSION	 = 0x0004
EFI_HOB_TYPE_END_OF_HOB_LIST = 0xffff

class EFI_HOB_GENERIC_HEADER(STRUCT):
	_fields_ = [
		('HobType',		UINT16),
		('HobLength',	UINT16),
		('Reserved',	UINT32)
	]

class EFI_HOB_GUID_TYPE(STRUCT):
	_fields_ = [
		('Header',	EFI_HOB_GENERIC_HEADER),
		('Name',	EFI_GUID)
	]

def GetHobList(ql) -> int:
	"""Get HOB list location in memory (ostensibly set by PEI).
	"""

	conftable_guid = ql.os.profile['HOB_LIST']['Guid']
	conftable_ptr = GetEfiConfigurationTable(ql.loader.dxe_context, conftable_guid)
	conftable = EFI_CONFIGURATION_TABLE.loadFrom(ql, conftable_ptr)

	return ql.unpack64(conftable.VendorTable)

def CreateHob(ql, hob) -> int:
	"""Add a HOB to the end of the HOB list.
	"""

	hoblist = GetHobList(ql)

	# look for the list end marker; uefi codebase assumes there is
	# always one
	while True:
		header = EFI_HOB_GENERIC_HEADER.loadFrom(ql, hoblist)

		if header.HobType == EFI_HOB_TYPE_END_OF_HOB_LIST:
			break

		hoblist += header.HobLength

	# overwrite end marker with the hob
	pHob = hoblist
	hob.saveTo(ql, pHob)
	hoblist += hob.sizeof()

	# create a new end marker istead, following the hob
	marker = EFI_HOB_GENERIC_HEADER()
	marker.HobType = EFI_HOB_TYPE_END_OF_HOB_LIST
	marker.HobLength = 0x0000
	marker.Reserved = 0x00000000
	marker.saveTo(ql, hoblist)

	# return the address the hob was written to; it might be useful
	return pHob



#!/usr/bin/env python3
# 
# Cross Platform and Multi Architecture Advanced Binary Emulation Framework
#

from qiling.os.const import *
from .const import *
from .utils import *
from .fncc import *
from .ProcessorBind import *
from .UefiSpec import *

@dxeapi(params={
	"a0": POINTER, #POINTER_T(struct_EFI_TIME)
	"a1": POINTER, #POINTER_T(struct_EFI_TIME_CAPABILITIES)
})
def hook_GetTime(ql, address, params):
	return EFI_SUCCESS

@dxeapi(params={
	"a0": POINTER, #POINTER_T(struct_EFI_TIME)
})
def hook_SetTime(ql, address, params):
	return EFI_SUCCESS

@dxeapi(params={
	"a0": POINTER, #POINTER_T(ctypes.c_ubyte)
	"a1": POINTER, #POINTER_T(ctypes.c_ubyte)
	"a2": POINTER, #POINTER_T(struct_EFI_TIME)
})
def hook_GetWakeupTime(ql, address, params):
	return EFI_SUCCESS

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": POINTER, #POINTER_T(struct_EFI_TIME)
})
def hook_SetWakeupTime(ql, address, params):
	return EFI_SUCCESS

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": ULONGLONG,
	"a2": UINT,
	"a3": POINTER, #POINTER_T(struct_EFI_MEMORY_DESCRIPTOR)
})
def hook_SetVirtualAddressMap(ql, address, params):
	return EFI_SUCCESS

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": POINTER, #POINTER_T(POINTER_T(None))
})
def hook_ConvertPointer(ql, address, params):
	return EFI_SUCCESS

@dxeapi(params={
	"VariableName": WSTRING,
	"VendorGuid": GUID,
	"Attributes": POINTER,
	"DataSize": POINTER,
	"Data": POINTER
})
def hook_GetVariable(ql, address, params):
	if params['VariableName'] in ql.env:
		var = ql.env[params['VariableName']]
		read_len = read_int64(ql, params['DataSize'])
		if params['Attributes'] != 0:
			write_int64(ql, params['Attributes'], 0)
		write_int64(ql, params['DataSize'], len(var))
		if read_len < len(var):
			return EFI_BUFFER_TOO_SMALL
		if params['Data'] != 0:
			ql.mem.write(params['Data'], var)
		return EFI_SUCCESS
	return EFI_NOT_FOUND

@dxeapi(params={
	"VariableNameSize": POINTER, #POINTER_T(ctypes.c_uint64)
	"VariableName": POINTER, #POINTER_T(ctypes.c_uint16)
	"VendorGuid": GUID,
})
def hook_GetNextVariableName(ql, address, params):
	name_size = read_int64(ql, params["VariableNameSize"])
	last_name = ql.os.read_wstring(params["VariableName"])
	vars = ql.env['Names'] # This is a list of variable names in correct order.
	if last_name in vars and vars.index(last_name) < len(vars) - 1:
		new_name = vars[vars.index(last_name)+1]
		if (len(new_name)+1)*2 > name_size:
			return EFI_BUFFER_TOO_SMALL
		vn_ptr = params["VariableName"]
		for char in new_name:
			ql.mem.write(vn_ptr, char)
			vn_ptr += 1
			ql.mem.write(vn_ptr, '\x00')
			vn_ptr += 1
		ql.mem.write(vn_ptr, '\x00\x00')

	return EFI_INVALID_PARAMETER

@dxeapi(params={
	"VariableName": WSTRING, #POINTER_T(ctypes.c_uint16)
	"VendorGuid": GUID,
	"Attributes": UINT,
	"DataSize": ULONGLONG,
	"Data": POINTER, #POINTER_T(None)
})
def hook_SetVariable(ql, address, params):
	ql.env[params['VariableName']] = bytes(ql.mem.read(params['Data'], params['DataSize']))
	return EFI_SUCCESS

@dxeapi(params={
	"Count": POINTER, #POINTER_T(ctypes.c_uint32)
})
def hook_GetNextHighMonotonicCount(ql, address, params):
	ql.os.monotonic_count += 0x0000000100000000
	hmc = ql.os.monotonic_count
	hmc = (hmc >> 32) & 0xffffffff
	write_int32(ql, params["Count"], hmc)
	return EFI_SUCCESS

@dxeapi(params={
	"a0": ULONGLONG,
	"a1": ULONGLONG,
	"a2": ULONGLONG,
	"a3": POINTER, #POINTER_T(None)
})
def hook_ResetSystem(ql, address, params):
	ql.emu_stop()

	return EFI_SUCCESS

@dxeapi(params={
	"a0": POINTER, #POINTER_T(POINTER_T(struct_EFI_CAPSULE_HEADER))
	"a1": ULONGLONG,
	"a2": ULONGLONG,
})
def hook_UpdateCapsule(ql, address, params):
	return EFI_SUCCESS

@dxeapi(params={
	"a0": POINTER, #POINTER_T(POINTER_T(struct_EFI_CAPSULE_HEADER))
	"a1": ULONGLONG,
	"a2": POINTER, #POINTER_T(ctypes.c_uint64)
	"a3": POINTER, #POINTER_T(enum_73)
})
def hook_QueryCapsuleCapabilities(ql, address, params):
	return EFI_SUCCESS

@dxeapi(params={
	"a0": UINT,
	"a1": POINTER, #POINTER_T(ctypes.c_uint64)
	"a2": POINTER, #POINTER_T(ctypes.c_uint64)
	"a3": POINTER, #POINTER_T(ctypes.c_uint64)
})
def hook_QueryVariableInfo(ql, address, params):
	return EFI_SUCCESS

def initialize(ql, gRT : int):
	descriptor = {
		'struct' : EFI_RUNTIME_SERVICES,
		'fields' : (
			('Hdr',							None),
			('GetTime',						hook_GetTime),
			('SetTime',						hook_SetTime),
			('GetWakeupTime',				hook_GetWakeupTime),
			('SetWakeupTime',				hook_SetWakeupTime),
			('SetVirtualAddressMap',		hook_SetVirtualAddressMap),
			('ConvertPointer',				hook_ConvertPointer),
			('GetVariable',					hook_GetVariable),
			('GetNextVariableName',			hook_GetNextVariableName),
			('SetVariable',					hook_SetVariable),
			('GetNextHighMonotonicCount',	hook_GetNextHighMonotonicCount),
			('ResetSystem',					hook_ResetSystem),
			('UpdateCapsule',				hook_UpdateCapsule),
			('QueryCapsuleCapabilities',	hook_QueryCapsuleCapabilities),
			('QueryVariableInfo',			hook_QueryVariableInfo)
		)
	}

	instance = init_struct(ql, gRT, descriptor)
	instance.saveTo(ql, gRT)

__all__ = [
	'initialize'
]